#ifndef __IR_d_ir__
#define __IR_d_ir__

#include <stddef.h>

#ifdef offsetof
#define _IR_offsetof(type, field) offsetof (type, field)
#else
#define _IR_offsetof(type, field) ((char *)&((type *) 64)->field - (char *) 64)
#endif


#line 32 "./d_ir.sprut"


#include "d_common.h"
#include "d_types.h"

extern int it_is_int_string (const char *str);
extern int_t a2i (const char *str);
extern floating_t a2f (const char *str);
extern const char *i2a (int_t number);
extern const char *f2a (floating_t number);

/* Structures representing binary_operation_node and unary_operation_node
   have member of its mode.  EVT is abbreviation of expr value type.
   The order of enumeration constants is important. */

enum expr_value_type
{
  /* To correctly initiate nodes' member value_type of this enumeration
     constant is to be equal to the following value. */
  EVT_UNKNOWN = 0,
  EVT_CHAR = 1,
  EVT_INT = 2,
  EVT_FLOAT = 4,
  EVT_VECTOR = 8,
  EVT_TABLE = 16,
  EVT_FUNC = 32,
  EVT_CLASS = 64,
  EVT_TYPE = 128
};

/* This is mask for number. */
#define EVT_NUMBER_MASK (EVT_CHAR | EVT_INT | EVT_FLOAT)
/* This is mask for number or vector. */
#define EVT_NUMBER_VECTOR_MASK (EVT_CHAR | EVT_INT | EVT_FLOAT | EVT_VECTOR)
/* This is mask for number, vector, or table. */
#define EVT_NUMBER_VECTOR_TABLE_MASK\
   (EVT_CHAR | EVT_INT | EVT_FLOAT | EVT_VECTOR  | EVT_TABLE)
/* This is mask for number or string. */
#define EVT_NUMBER_STRING_MASK EVT_NUMBER_VECTOR_MASK

typedef enum expr_value_type expr_value_type_t;

typedef int type_mask_t;

typedef int integer_t;

typedef char *string_t;

typedef void (*implementation_func_t) (int_t pars_number);



#line 67 "d_ir.h"

typedef enum IR_node_mode_enum
{
  IR_NM_ir_node,
  IR_NM_unique_char,
  IR_NM_generic_pos,
  IR_NM_char,
  IR_NM_unique_int,
  IR_NM_int,
  IR_NM_unique_float,
  IR_NM_float,
  IR_NM_unique_string,
  IR_NM_string,
  IR_NM_unique_ident,
  IR_NM_ident,
  IR_NM_expr,
  IR_NM_operation,
  IR_NM_binary_operation,
  IR_NM_unary_operation,
  IR_NM_in,
  IR_NM_or,
  IR_NM_xor,
  IR_NM_and,
  IR_NM_eq,
  IR_NM_ne,
  IR_NM_identity,
  IR_NM_unidentity,
  IR_NM_lt,
  IR_NM_gt,
  IR_NM_le,
  IR_NM_ge,
  IR_NM_lshift,
  IR_NM_rshift,
  IR_NM_ashift,
  IR_NM_concat,
  IR_NM_plus,
  IR_NM_minus,
  IR_NM_mult,
  IR_NM_div,
  IR_NM_mod,
  IR_NM_index,
  IR_NM_key_index,
  IR_NM_member_access,
  IR_NM_period,
  IR_NM_logical_binary_operation,
  IR_NM_logical_or,
  IR_NM_logical_and,
  IR_NM_cond,
  IR_NM_const,
  IR_NM_new,
  IR_NM_not,
  IR_NM_unary_plus,
  IR_NM_unary_minus,
  IR_NM_bitwise_not,
  IR_NM_length,
  IR_NM_typeof,
  IR_NM_charof,
  IR_NM_intof,
  IR_NM_floatof,
  IR_NM_vectorof,
  IR_NM_tableof,
  IR_NM_funcof,
  IR_NM_threadof,
  IR_NM_classof,
  IR_NM_format_vectorof,
  IR_NM_designator,
  IR_NM_type,
  IR_NM_nil_type,
  IR_NM_hide_type,
  IR_NM_hideblock_type,
  IR_NM_char_type,
  IR_NM_int_type,
  IR_NM_float_type,
  IR_NM_vector_type,
  IR_NM_table_type,
  IR_NM_func_type,
  IR_NM_thread_type,
  IR_NM_class_type,
  IR_NM_stack_type,
  IR_NM_process_type,
  IR_NM_instance_type,
  IR_NM_type_type,
  IR_NM_vector_table,
  IR_NM_vector,
  IR_NM_table,
  IR_NM_elist_element,
  IR_NM_class_func_thread_call,
  IR_NM_access_ident,
  IR_NM_stmt,
  IR_NM_assign_stmt,
  IR_NM_assign,
  IR_NM_mult_assign,
  IR_NM_div_assign,
  IR_NM_rem_assign,
  IR_NM_plus_assign,
  IR_NM_minus_assign,
  IR_NM_concat_assign,
  IR_NM_lshift_assign,
  IR_NM_rshift_assign,
  IR_NM_ashift_assign,
  IR_NM_and_assign,
  IR_NM_xor_assign,
  IR_NM_or_assign,
  IR_NM_var_assign,
  IR_NM_par_assign,
  IR_NM_swap,
  IR_NM_procedure_call,
  IR_NM_if_stmt,
  IR_NM_for_stmt,
  IR_NM_foreach_stmt,
  IR_NM_for_break_contintue,
  IR_NM_break_stmt,
  IR_NM_continue_stmt,
  IR_NM_return_without_result,
  IR_NM_return_with_result,
  IR_NM_throw,
  IR_NM_wait_stmt,
  IR_NM_block,
  IR_NM_exception,
  IR_NM_decl,
  IR_NM_may_be_const_decl,
  IR_NM_var,
  IR_NM_external_var,
  IR_NM_pub,
  IR_NM_external_func,
  IR_NM_func_class_ext,
  IR_NM_func_or_class,
  IR_NM_func,
  IR_NM_class,
  IR_NM_ext,
  IR_NM_no_testing_period,
  IR_NM_lvalue_period,
  IR_NM_lvalue_period_and_val,
  IR_NM_lvalue_index,
  IR_NM_lvalue_index_and_val,
  IR_NM_lvalue_key_index,
  IR_NM_lvalue_key_index_and_val,
  IR_NM_lvalue_no_testing_period,
  IR_NM_lvalue_no_testing_period_and_val,
  IR_NM_logical_or_end,
  IR_NM_logical_and_end,
  IR_NM_cond_end,
  IR_NM_par_assign_end,
  IR_NM_par_assign_test,
  IR_NM_nil,
  IR_NM_friend,
  IR_NM_decl_in_block,
  IR_NM_occurrence,
  IR_NM_external_var_occurrence,
  IR_NM_local_var_occurrence,
  IR_NM_var_occurrence,
  IR_NM_lvalue_external_var_occurrence,
  IR_NM_lvalue_external_var_occurrence_and_val,
  IR_NM_local_lvalue_var_occurrence,
  IR_NM_local_lvalue_var_occurrence_and_val,
  IR_NM_lvalue_var_occurrence,
  IR_NM_lvalue_var_occurrence_and_val,
  IR_NM_external_func_occurrence,
  IR_NM_func_occurrence,
  IR_NM_class_occurrence,
  IR_NM_if_finish,
  IR_NM_for_finish,
  IR_NM_wait_finish,
  IR_NM_foreach_start,
  IR_NM_foreach_next_iteration,
  IR_NM_block_finish,
  IR_NM_catches_finish,
  IR_NM_pop_func_call,
  IR_NM_external,
  IR_NM__root,
  IR_NM__error
} IR_node_mode_t;

extern short _IR_node_level [];

extern unsigned char _IR_SF_ir_node [];

extern unsigned char _IR_SF_unique_char [];

extern unsigned char _IR_SF_generic_pos [];

extern unsigned char _IR_SF_char [];

extern unsigned char _IR_SF_unique_int [];

extern unsigned char _IR_SF_int [];

extern unsigned char _IR_SF_unique_float [];

extern unsigned char _IR_SF_float [];

extern unsigned char _IR_SF_unique_string [];

extern unsigned char _IR_SF_string [];

extern unsigned char _IR_SF_unique_ident [];

extern unsigned char _IR_SF_ident [];

extern unsigned char _IR_SF_expr [];

extern unsigned char _IR_SF_operation [];

extern unsigned char _IR_SF_binary_operation [];

extern unsigned char _IR_SF_unary_operation [];

extern unsigned char _IR_SF_in [];

extern unsigned char _IR_SF_or [];

extern unsigned char _IR_SF_xor [];

extern unsigned char _IR_SF_and [];

extern unsigned char _IR_SF_eq [];

extern unsigned char _IR_SF_ne [];

extern unsigned char _IR_SF_identity [];

extern unsigned char _IR_SF_unidentity [];

extern unsigned char _IR_SF_lt [];

extern unsigned char _IR_SF_gt [];

extern unsigned char _IR_SF_le [];

extern unsigned char _IR_SF_ge [];

extern unsigned char _IR_SF_lshift [];

extern unsigned char _IR_SF_rshift [];

extern unsigned char _IR_SF_ashift [];

extern unsigned char _IR_SF_concat [];

extern unsigned char _IR_SF_plus [];

extern unsigned char _IR_SF_minus [];

extern unsigned char _IR_SF_mult [];

extern unsigned char _IR_SF_div [];

extern unsigned char _IR_SF_mod [];

extern unsigned char _IR_SF_index [];

extern unsigned char _IR_SF_key_index [];

extern unsigned char _IR_SF_member_access [];

extern unsigned char _IR_SF_period [];

extern unsigned char _IR_SF_logical_binary_operation [];

extern unsigned char _IR_SF_logical_or [];

extern unsigned char _IR_SF_logical_and [];

extern unsigned char _IR_SF_cond [];

extern unsigned char _IR_SF_const [];

extern unsigned char _IR_SF_new [];

extern unsigned char _IR_SF_not [];

extern unsigned char _IR_SF_unary_plus [];

extern unsigned char _IR_SF_unary_minus [];

extern unsigned char _IR_SF_bitwise_not [];

extern unsigned char _IR_SF_length [];

extern unsigned char _IR_SF_typeof [];

extern unsigned char _IR_SF_charof [];

extern unsigned char _IR_SF_intof [];

extern unsigned char _IR_SF_floatof [];

extern unsigned char _IR_SF_vectorof [];

extern unsigned char _IR_SF_tableof [];

extern unsigned char _IR_SF_funcof [];

extern unsigned char _IR_SF_threadof [];

extern unsigned char _IR_SF_classof [];

extern unsigned char _IR_SF_format_vectorof [];

extern unsigned char _IR_SF_designator [];

extern unsigned char _IR_SF_type [];

extern unsigned char _IR_SF_nil_type [];

extern unsigned char _IR_SF_hide_type [];

extern unsigned char _IR_SF_hideblock_type [];

extern unsigned char _IR_SF_char_type [];

extern unsigned char _IR_SF_int_type [];

extern unsigned char _IR_SF_float_type [];

extern unsigned char _IR_SF_vector_type [];

extern unsigned char _IR_SF_table_type [];

extern unsigned char _IR_SF_func_type [];

extern unsigned char _IR_SF_thread_type [];

extern unsigned char _IR_SF_class_type [];

extern unsigned char _IR_SF_stack_type [];

extern unsigned char _IR_SF_process_type [];

extern unsigned char _IR_SF_instance_type [];

extern unsigned char _IR_SF_type_type [];

extern unsigned char _IR_SF_vector_table [];

extern unsigned char _IR_SF_vector [];

extern unsigned char _IR_SF_table [];

extern unsigned char _IR_SF_elist_element [];

extern unsigned char _IR_SF_class_func_thread_call [];

extern unsigned char _IR_SF_access_ident [];

extern unsigned char _IR_SF_stmt [];

extern unsigned char _IR_SF_assign_stmt [];

extern unsigned char _IR_SF_assign [];

extern unsigned char _IR_SF_mult_assign [];

extern unsigned char _IR_SF_div_assign [];

extern unsigned char _IR_SF_rem_assign [];

extern unsigned char _IR_SF_plus_assign [];

extern unsigned char _IR_SF_minus_assign [];

extern unsigned char _IR_SF_concat_assign [];

extern unsigned char _IR_SF_lshift_assign [];

extern unsigned char _IR_SF_rshift_assign [];

extern unsigned char _IR_SF_ashift_assign [];

extern unsigned char _IR_SF_and_assign [];

extern unsigned char _IR_SF_xor_assign [];

extern unsigned char _IR_SF_or_assign [];

extern unsigned char _IR_SF_var_assign [];

extern unsigned char _IR_SF_par_assign [];

extern unsigned char _IR_SF_swap [];

extern unsigned char _IR_SF_procedure_call [];

extern unsigned char _IR_SF_if_stmt [];

extern unsigned char _IR_SF_for_stmt [];

extern unsigned char _IR_SF_foreach_stmt [];

extern unsigned char _IR_SF_for_break_contintue [];

extern unsigned char _IR_SF_break_stmt [];

extern unsigned char _IR_SF_continue_stmt [];

extern unsigned char _IR_SF_return_without_result [];

extern unsigned char _IR_SF_return_with_result [];

extern unsigned char _IR_SF_throw [];

extern unsigned char _IR_SF_wait_stmt [];

extern unsigned char _IR_SF_block [];

extern unsigned char _IR_SF_exception [];

extern unsigned char _IR_SF_decl [];

extern unsigned char _IR_SF_may_be_const_decl [];

extern unsigned char _IR_SF_var [];

extern unsigned char _IR_SF_external_var [];

extern unsigned char _IR_SF_pub [];

extern unsigned char _IR_SF_external_func [];

extern unsigned char _IR_SF_func_class_ext [];

extern unsigned char _IR_SF_func_or_class [];

extern unsigned char _IR_SF_func [];

extern unsigned char _IR_SF_class [];

extern unsigned char _IR_SF_ext [];

extern unsigned char _IR_SF_no_testing_period [];

extern unsigned char _IR_SF_lvalue_period [];

extern unsigned char _IR_SF_lvalue_period_and_val [];

extern unsigned char _IR_SF_lvalue_index [];

extern unsigned char _IR_SF_lvalue_index_and_val [];

extern unsigned char _IR_SF_lvalue_key_index [];

extern unsigned char _IR_SF_lvalue_key_index_and_val [];

extern unsigned char _IR_SF_lvalue_no_testing_period [];

extern unsigned char _IR_SF_lvalue_no_testing_period_and_val [];

extern unsigned char _IR_SF_logical_or_end [];

extern unsigned char _IR_SF_logical_and_end [];

extern unsigned char _IR_SF_cond_end [];

extern unsigned char _IR_SF_par_assign_end [];

extern unsigned char _IR_SF_par_assign_test [];

extern unsigned char _IR_SF_nil [];

extern unsigned char _IR_SF_friend [];

extern unsigned char _IR_SF_decl_in_block [];

extern unsigned char _IR_SF_occurrence [];

extern unsigned char _IR_SF_external_var_occurrence [];

extern unsigned char _IR_SF_local_var_occurrence [];

extern unsigned char _IR_SF_var_occurrence [];

extern unsigned char _IR_SF_lvalue_external_var_occurrence [];

extern unsigned char _IR_SF_lvalue_external_var_occurrence_and_val [];

extern unsigned char _IR_SF_local_lvalue_var_occurrence [];

extern unsigned char _IR_SF_local_lvalue_var_occurrence_and_val [];

extern unsigned char _IR_SF_lvalue_var_occurrence [];

extern unsigned char _IR_SF_lvalue_var_occurrence_and_val [];

extern unsigned char _IR_SF_external_func_occurrence [];

extern unsigned char _IR_SF_func_occurrence [];

extern unsigned char _IR_SF_class_occurrence [];

extern unsigned char _IR_SF_if_finish [];

extern unsigned char _IR_SF_for_finish [];

extern unsigned char _IR_SF_wait_finish [];

extern unsigned char _IR_SF_foreach_start [];

extern unsigned char _IR_SF_foreach_next_iteration [];

extern unsigned char _IR_SF_block_finish [];

extern unsigned char _IR_SF_catches_finish [];

extern unsigned char _IR_SF_pop_func_call [];

extern unsigned char _IR_SF_external [];

extern unsigned char _IR_SF__root [];

extern unsigned char _IR_SF__error [];

extern unsigned char *_IR_is_type[];

extern unsigned char _IR_D_ident [];

extern unsigned char _IR_D_catch_list_pc [];

extern unsigned char _IR_D_no [];

extern unsigned char _IR_D_decl_number_in_block [];

extern unsigned char _IR_D_address [];

extern void *_IR_class_structure_array [];

typedef struct IR_node *IR_node_t;

typedef struct _IR_double_link *IR_double_link_t;

struct _IR_double_link
{
  IR_node_t field_itself;
  IR_node_t link_owner;
  void (*set_function) (IR_node_t, IR_node_t);
  IR_double_link_t previous_link;
  IR_double_link_t next_link;
};

struct _IR_S_ir_node
{
  IR_node_t  next_pc;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_ir_node _IR_S_ir_node;
} _IR_ir_node;

struct _IR_S_unique_char
{
  struct _IR_S_ir_node _IR_M_ir_node;
  char_t  char_value;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_unique_char _IR_S_unique_char;
} _IR_unique_char;

struct _IR_S_generic_pos
{
  struct _IR_S_ir_node _IR_M_ir_node;
  position_t  pos;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_generic_pos _IR_S_generic_pos;
} _IR_generic_pos;

typedef struct
{
  IR_node_mode_t _IR_node_mode;
} _IR_expr;

struct _IR_S_char
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
  IR_node_t  unique_char;
  char_t  ch_val;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_char _IR_S_char;
} _IR_char;

struct _IR_S_unique_int
{
  struct _IR_S_ir_node _IR_M_ir_node;
  int_t  int_value;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_unique_int _IR_S_unique_int;
} _IR_unique_int;

struct _IR_S_int
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
  IR_node_t  unique_int;
  int_t  i_val;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_int _IR_S_int;
} _IR_int;

struct _IR_S_unique_float
{
  struct _IR_S_ir_node _IR_M_ir_node;
  floating_t  float_value;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_unique_float _IR_S_unique_float;
} _IR_unique_float;

struct _IR_S_float
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
  IR_node_t  unique_float;
  floating_t  f_val;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_float _IR_S_float;
} _IR_float;

struct _IR_S_unique_string
{
  struct _IR_S_ir_node _IR_M_ir_node;
  string_t  string_value;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_unique_string _IR_S_unique_string;
} _IR_unique_string;

struct _IR_S_string
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
  IR_node_t  unique_string;
  string_t  str_val;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_string _IR_S_string;
} _IR_string;

struct _IR_S_unique_ident
{
  struct _IR_S_ir_node _IR_M_ir_node;
  string_t  ident_string;
  integer_t  block_decl_ident_number;
  bool_t  it_is_declared_in_block;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_unique_ident _IR_S_unique_ident;
} _IR_unique_ident;

typedef struct
{
  IR_node_mode_t _IR_node_mode;
} _IR_designator;

struct _IR_S_ident
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
  IR_node_t  unique_ident;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_ident _IR_S_ident;
} _IR_ident;

struct _IR_S_operation
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
  type_mask_t  value_type;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_operation _IR_S_operation;
} _IR_operation;

struct _IR_S_binary_operation
{
  struct _IR_S_operation _IR_M_operation;
  IR_node_t  left_operand;
  IR_node_t  right_operand;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_binary_operation _IR_S_binary_operation;
} _IR_binary_operation;

struct _IR_S_unary_operation
{
  struct _IR_S_operation _IR_M_operation;
  IR_node_t  operand;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_unary_operation _IR_S_unary_operation;
} _IR_unary_operation;

struct _IR_S_in
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_in _IR_S_in;
} _IR_in;

struct _IR_S_or
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_or _IR_S_or;
} _IR_or;

struct _IR_S_xor
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_xor _IR_S_xor;
} _IR_xor;

struct _IR_S_and
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_and _IR_S_and;
} _IR_and;

struct _IR_S_eq
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_eq _IR_S_eq;
} _IR_eq;

struct _IR_S_ne
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_ne _IR_S_ne;
} _IR_ne;

struct _IR_S_identity
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_identity _IR_S_identity;
} _IR_identity;

struct _IR_S_unidentity
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_unidentity _IR_S_unidentity;
} _IR_unidentity;

struct _IR_S_lt
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_lt _IR_S_lt;
} _IR_lt;

struct _IR_S_gt
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_gt _IR_S_gt;
} _IR_gt;

struct _IR_S_le
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_le _IR_S_le;
} _IR_le;

struct _IR_S_ge
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_ge _IR_S_ge;
} _IR_ge;

struct _IR_S_lshift
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_lshift _IR_S_lshift;
} _IR_lshift;

struct _IR_S_rshift
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_rshift _IR_S_rshift;
} _IR_rshift;

struct _IR_S_ashift
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_ashift _IR_S_ashift;
} _IR_ashift;

struct _IR_S_concat
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_concat _IR_S_concat;
} _IR_concat;

struct _IR_S_plus
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_plus _IR_S_plus;
} _IR_plus;

struct _IR_S_minus
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_minus _IR_S_minus;
} _IR_minus;

struct _IR_S_mult
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_mult _IR_S_mult;
} _IR_mult;

struct _IR_S_div
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_div _IR_S_div;
} _IR_div;

struct _IR_S_mod
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_mod _IR_S_mod;
} _IR_mod;

struct _IR_S_index
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_index _IR_S_index;
} _IR_index;

struct _IR_S_key_index
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_key_index _IR_S_key_index;
} _IR_key_index;

struct _IR_S_member_access
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
  integer_t  right_block_decl_ident_number;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_member_access _IR_S_member_access;
} _IR_member_access;

struct _IR_S_period
{
  struct _IR_S_member_access _IR_M_member_access;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_period _IR_S_period;
} _IR_period;

struct _IR_S_logical_binary_operation
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
  IR_node_t  short_path_pc;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_logical_binary_operation _IR_S_logical_binary_operation;
} _IR_logical_binary_operation;

struct _IR_S_logical_or
{
  struct _IR_S_logical_binary_operation _IR_M_logical_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_logical_or _IR_S_logical_or;
} _IR_logical_or;

struct _IR_S_logical_and
{
  struct _IR_S_logical_binary_operation _IR_M_logical_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_logical_and _IR_S_logical_and;
} _IR_logical_and;

struct _IR_S_cond
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
  IR_node_t  cond;
  IR_node_t  false_path_pc;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_cond _IR_S_cond;
} _IR_cond;

struct _IR_S_const
{
  struct _IR_S_unary_operation _IR_M_unary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_const _IR_S_const;
} _IR_const;

struct _IR_S_new
{
  struct _IR_S_unary_operation _IR_M_unary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_new _IR_S_new;
} _IR_new;

struct _IR_S_not
{
  struct _IR_S_unary_operation _IR_M_unary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_not _IR_S_not;
} _IR_not;

struct _IR_S_unary_plus
{
  struct _IR_S_unary_operation _IR_M_unary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_unary_plus _IR_S_unary_plus;
} _IR_unary_plus;

struct _IR_S_unary_minus
{
  struct _IR_S_unary_operation _IR_M_unary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_unary_minus _IR_S_unary_minus;
} _IR_unary_minus;

struct _IR_S_bitwise_not
{
  struct _IR_S_unary_operation _IR_M_unary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_bitwise_not _IR_S_bitwise_not;
} _IR_bitwise_not;

struct _IR_S_length
{
  struct _IR_S_unary_operation _IR_M_unary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_length _IR_S_length;
} _IR_length;

struct _IR_S_typeof
{
  struct _IR_S_unary_operation _IR_M_unary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_typeof _IR_S_typeof;
} _IR_typeof;

struct _IR_S_charof
{
  struct _IR_S_unary_operation _IR_M_unary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_charof _IR_S_charof;
} _IR_charof;

struct _IR_S_intof
{
  struct _IR_S_unary_operation _IR_M_unary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_intof _IR_S_intof;
} _IR_intof;

struct _IR_S_floatof
{
  struct _IR_S_unary_operation _IR_M_unary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_floatof _IR_S_floatof;
} _IR_floatof;

struct _IR_S_vectorof
{
  struct _IR_S_unary_operation _IR_M_unary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_vectorof _IR_S_vectorof;
} _IR_vectorof;

struct _IR_S_tableof
{
  struct _IR_S_unary_operation _IR_M_unary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_tableof _IR_S_tableof;
} _IR_tableof;

struct _IR_S_funcof
{
  struct _IR_S_unary_operation _IR_M_unary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_funcof _IR_S_funcof;
} _IR_funcof;

struct _IR_S_threadof
{
  struct _IR_S_unary_operation _IR_M_unary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_threadof _IR_S_threadof;
} _IR_threadof;

struct _IR_S_classof
{
  struct _IR_S_unary_operation _IR_M_unary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_classof _IR_S_classof;
} _IR_classof;

struct _IR_S_format_vectorof
{
  struct _IR_S_vectorof _IR_M_vectorof;
  IR_node_t  format;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_format_vectorof _IR_S_format_vectorof;
} _IR_format_vectorof;

struct _IR_S_type
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_type _IR_S_type;
} _IR_type;

struct _IR_S_nil_type
{
  struct _IR_S_type _IR_M_type;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_nil_type _IR_S_nil_type;
} _IR_nil_type;

struct _IR_S_hide_type
{
  struct _IR_S_type _IR_M_type;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_hide_type _IR_S_hide_type;
} _IR_hide_type;

struct _IR_S_hideblock_type
{
  struct _IR_S_type _IR_M_type;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_hideblock_type _IR_S_hideblock_type;
} _IR_hideblock_type;

struct _IR_S_char_type
{
  struct _IR_S_type _IR_M_type;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_char_type _IR_S_char_type;
} _IR_char_type;

struct _IR_S_int_type
{
  struct _IR_S_type _IR_M_type;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_int_type _IR_S_int_type;
} _IR_int_type;

struct _IR_S_float_type
{
  struct _IR_S_type _IR_M_type;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_float_type _IR_S_float_type;
} _IR_float_type;

struct _IR_S_vector_type
{
  struct _IR_S_type _IR_M_type;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_vector_type _IR_S_vector_type;
} _IR_vector_type;

struct _IR_S_table_type
{
  struct _IR_S_type _IR_M_type;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_table_type _IR_S_table_type;
} _IR_table_type;

struct _IR_S_func_type
{
  struct _IR_S_type _IR_M_type;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_func_type _IR_S_func_type;
} _IR_func_type;

struct _IR_S_thread_type
{
  struct _IR_S_type _IR_M_type;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_thread_type _IR_S_thread_type;
} _IR_thread_type;

struct _IR_S_class_type
{
  struct _IR_S_type _IR_M_type;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_class_type _IR_S_class_type;
} _IR_class_type;

struct _IR_S_stack_type
{
  struct _IR_S_type _IR_M_type;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_stack_type _IR_S_stack_type;
} _IR_stack_type;

struct _IR_S_process_type
{
  struct _IR_S_type _IR_M_type;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_process_type _IR_S_process_type;
} _IR_process_type;

struct _IR_S_instance_type
{
  struct _IR_S_type _IR_M_type;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_instance_type _IR_S_instance_type;
} _IR_instance_type;

struct _IR_S_type_type
{
  struct _IR_S_type _IR_M_type;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_type_type _IR_S_type_type;
} _IR_type_type;

struct _IR_S_vector_table
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
  IR_node_t  elist;
  integer_t  parts_number;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_vector_table _IR_S_vector_table;
} _IR_vector_table;

struct _IR_S_vector
{
  struct _IR_S_vector_table _IR_M_vector_table;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_vector _IR_S_vector;
} _IR_vector;

struct _IR_S_table
{
  struct _IR_S_vector_table _IR_M_vector_table;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_table _IR_S_table;
} _IR_table;

struct _IR_S_elist_element
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
  IR_node_t  repetition_key;
  IR_node_t  expr;
  IR_node_t  next_elist;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_elist_element _IR_S_elist_element;
} _IR_elist_element;

struct _IR_S_class_func_thread_call
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
  IR_node_t  func_expr;
  IR_node_t  actuals;
  integer_t  class_func_thread_call_parameters_number;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_class_func_thread_call _IR_S_class_func_thread_call;
} _IR_class_func_thread_call;

struct _IR_S_access_ident
{
  struct _IR_S_ir_node _IR_M_ir_node;
  IR_node_t  ident;
  bool_t  access_ident_public_flag;
  bool_t  friend_flag;
  IR_node_t  next_access_ident;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_access_ident _IR_S_access_ident;
} _IR_access_ident;

struct _IR_S_stmt
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
  IR_node_t  next_stmt;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_stmt _IR_S_stmt;
} _IR_stmt;

struct _IR_S_assign_stmt
{
  struct _IR_S_stmt _IR_M_stmt;
  IR_node_t  assignment_var;
  IR_node_t  assignment_expr;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_assign_stmt _IR_S_assign_stmt;
} _IR_assign_stmt;

struct _IR_S_assign
{
  struct _IR_S_assign_stmt _IR_M_assign_stmt;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_assign _IR_S_assign;
} _IR_assign;

struct _IR_S_mult_assign
{
  struct _IR_S_assign_stmt _IR_M_assign_stmt;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_mult_assign _IR_S_mult_assign;
} _IR_mult_assign;

struct _IR_S_div_assign
{
  struct _IR_S_assign_stmt _IR_M_assign_stmt;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_div_assign _IR_S_div_assign;
} _IR_div_assign;

struct _IR_S_rem_assign
{
  struct _IR_S_assign_stmt _IR_M_assign_stmt;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_rem_assign _IR_S_rem_assign;
} _IR_rem_assign;

struct _IR_S_plus_assign
{
  struct _IR_S_assign_stmt _IR_M_assign_stmt;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_plus_assign _IR_S_plus_assign;
} _IR_plus_assign;

struct _IR_S_minus_assign
{
  struct _IR_S_assign_stmt _IR_M_assign_stmt;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_minus_assign _IR_S_minus_assign;
} _IR_minus_assign;

struct _IR_S_concat_assign
{
  struct _IR_S_assign_stmt _IR_M_assign_stmt;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_concat_assign _IR_S_concat_assign;
} _IR_concat_assign;

struct _IR_S_lshift_assign
{
  struct _IR_S_assign_stmt _IR_M_assign_stmt;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_lshift_assign _IR_S_lshift_assign;
} _IR_lshift_assign;

struct _IR_S_rshift_assign
{
  struct _IR_S_assign_stmt _IR_M_assign_stmt;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_rshift_assign _IR_S_rshift_assign;
} _IR_rshift_assign;

struct _IR_S_ashift_assign
{
  struct _IR_S_assign_stmt _IR_M_assign_stmt;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_ashift_assign _IR_S_ashift_assign;
} _IR_ashift_assign;

struct _IR_S_and_assign
{
  struct _IR_S_assign_stmt _IR_M_assign_stmt;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_and_assign _IR_S_and_assign;
} _IR_and_assign;

struct _IR_S_xor_assign
{
  struct _IR_S_assign_stmt _IR_M_assign_stmt;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_xor_assign _IR_S_xor_assign;
} _IR_xor_assign;

struct _IR_S_or_assign
{
  struct _IR_S_assign_stmt _IR_M_assign_stmt;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_or_assign _IR_S_or_assign;
} _IR_or_assign;

struct _IR_S_var_assign
{
  struct _IR_S_assign_stmt _IR_M_assign_stmt;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_var_assign _IR_S_var_assign;
} _IR_var_assign;

struct _IR_S_par_assign
{
  struct _IR_S_assign_stmt _IR_M_assign_stmt;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_par_assign _IR_S_par_assign;
} _IR_par_assign;

struct _IR_S_swap
{
  struct _IR_S_stmt _IR_M_stmt;
  IR_node_t  swap_var1;
  IR_node_t  swap_var2;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_swap _IR_S_swap;
} _IR_swap;

struct _IR_S_procedure_call
{
  struct _IR_S_stmt _IR_M_stmt;
  IR_node_t  proc_expr;
  IR_node_t  proc_actuals;
  integer_t  procedure_call_pars_number;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_procedure_call _IR_S_procedure_call;
} _IR_procedure_call;

struct _IR_S_if_stmt
{
  struct _IR_S_stmt _IR_M_stmt;
  IR_node_t  if_expr;
  IR_node_t  if_part;
  IR_node_t  else_part;
  IR_node_t  else_part_pc;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_if_stmt _IR_S_if_stmt;
} _IR_if_stmt;

struct _IR_S_for_stmt
{
  struct _IR_S_stmt _IR_M_stmt;
  IR_node_t  for_initial_stmt;
  IR_node_t  for_guard_expr;
  IR_node_t  for_iterate_stmt;
  IR_node_t  for_stmts;
  IR_node_t  for_body_pc;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_for_stmt _IR_S_for_stmt;
} _IR_for_stmt;

struct _IR_S_foreach_stmt
{
  struct _IR_S_stmt _IR_M_stmt;
  IR_node_t  foreach_designator;
  IR_node_t  foreach_table;
  IR_node_t  foreach_stmts;
  IR_node_t  foreach_body_pc;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_foreach_stmt _IR_S_foreach_stmt;
} _IR_foreach_stmt;

struct _IR_S_for_break_contintue
{
  struct _IR_S_stmt _IR_M_stmt;
  integer_t  number_of_surrounding_blocks;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_for_break_contintue _IR_S_for_break_contintue;
} _IR_for_break_contintue;

struct _IR_S_break_stmt
{
  struct _IR_S_for_break_contintue _IR_M_for_break_contintue;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_break_stmt _IR_S_break_stmt;
} _IR_break_stmt;

struct _IR_S_continue_stmt
{
  struct _IR_S_for_break_contintue _IR_M_for_break_contintue;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_continue_stmt _IR_S_continue_stmt;
} _IR_continue_stmt;

struct _IR_S_return_without_result
{
  struct _IR_S_stmt _IR_M_stmt;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_return_without_result _IR_S_return_without_result;
} _IR_return_without_result;

struct _IR_S_return_with_result
{
  struct _IR_S_stmt _IR_M_stmt;
  IR_node_t  returned_expr;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_return_with_result _IR_S_return_with_result;
} _IR_return_with_result;

struct _IR_S_throw
{
  struct _IR_S_stmt _IR_M_stmt;
  IR_node_t  throw_expr;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_throw _IR_S_throw;
} _IR_throw;

struct _IR_S_wait_stmt
{
  struct _IR_S_stmt _IR_M_stmt;
  IR_node_t  wait_guard_expr;
  IR_node_t  wait_stmt;
  IR_node_t  start_wait_guard_expr_pc;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_wait_stmt _IR_S_wait_stmt;
} _IR_wait_stmt;

struct _IR_S_block
{
  struct _IR_S_stmt _IR_M_stmt;
  IR_node_t  block_scope;
  IR_node_t  block_stmts;
  IR_node_t  func_class_ext;
  IR_node_t  access_list;
  IR_node_t  exceptions;
  integer_t  block_level;
  integer_t  decls_number;
  integer_t  vars_number;
  integer_t  temporary_vars_number;
  integer_t  block_number;
  IR_node_t  catch_list_pc;
  bool_t  simple_block_flag;
  bool_t  extended_life_context_flag;
  IR_node_t  friends;
  string_t  cached_container_address;
  integer_t  cached_container_tick;
  bool_t  block_saved_sync_flag;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_block _IR_S_block;
} _IR_block;

struct _IR_S_exception
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
  IR_node_t  exception_class_expr;
  IR_node_t  catch_block;
  IR_node_t  next_exception;
  IR_node_t  catch_list_pc;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_exception _IR_S_exception;
} _IR_exception;

struct _IR_S_decl
{
  struct _IR_S_stmt _IR_M_stmt;
  IR_node_t  scope;
  IR_node_t  ident;
  implementation_func_t  implementation_func;
  bool_t  public_flag;
  IR_node_t  access_ident;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_decl _IR_S_decl;
} _IR_decl;

struct _IR_S_may_be_const_decl
{
  struct _IR_S_decl _IR_M_decl;
  bool_t  const_flag;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_may_be_const_decl _IR_S_may_be_const_decl;
} _IR_may_be_const_decl;

struct _IR_S_decl_in_block
{
  integer_t  decl_number_in_block;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_decl_in_block _IR_S_decl_in_block;
} _IR_decl_in_block;

struct _IR_S_var
{
  struct _IR_S_may_be_const_decl _IR_M_may_be_const_decl;
  struct _IR_S_decl_in_block _IR_M_decl_in_block;
  integer_t  var_number_in_block;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_var _IR_S_var;
} _IR_var;

struct _IR_S_external
{
  string_t  address;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_external _IR_S_external;
} _IR_external;

struct _IR_S_external_var
{
  struct _IR_S_may_be_const_decl _IR_M_may_be_const_decl;
  struct _IR_S_decl_in_block _IR_M_decl_in_block;
  struct _IR_S_external _IR_M_external;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_external_var _IR_S_external_var;
} _IR_external_var;

struct _IR_S_pub
{
  struct _IR_S_decl _IR_M_decl;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_pub _IR_S_pub;
} _IR_pub;

struct _IR_S_external_func
{
  struct _IR_S_decl _IR_M_decl;
  struct _IR_S_decl_in_block _IR_M_decl_in_block;
  struct _IR_S_external _IR_M_external;
  int_t  no;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_external_func _IR_S_external_func;
} _IR_external_func;

struct _IR_S_func_class_ext
{
  struct _IR_S_decl _IR_M_decl;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_func_class_ext _IR_S_func_class_ext;
} _IR_func_class_ext;

struct _IR_S_func_or_class
{
  struct _IR_S_func_class_ext _IR_M_func_class_ext;
  struct _IR_S_decl_in_block _IR_M_decl_in_block;
  struct _IR_S_external _IR_M_external;
  integer_t  parameters_number;
  bool_t  args_flag;
  bool_t  final_flag;
  int_t  no;
  integer_t  calls_number;
  ticker_t  exec_time;
  integer_t  interrupts_number;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_func_or_class _IR_S_func_or_class;
} _IR_func_or_class;

struct _IR_S_func
{
  struct _IR_S_func_or_class _IR_M_func_or_class;
  bool_t  thread_flag;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_func _IR_S_func;
} _IR_func;

struct _IR_S_class
{
  struct _IR_S_func_or_class _IR_M_func_or_class;
  bool_t  simple_class_flag;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_class _IR_S_class;
} _IR_class;

struct _IR_S_ext
{
  struct _IR_S_func_class_ext _IR_M_func_class_ext;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_ext _IR_S_ext;
} _IR_ext;

struct _IR_S_no_testing_period
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_no_testing_period _IR_S_no_testing_period;
} _IR_no_testing_period;

struct _IR_S_lvalue_period
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_lvalue_period _IR_S_lvalue_period;
} _IR_lvalue_period;

struct _IR_S_lvalue_period_and_val
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_lvalue_period_and_val _IR_S_lvalue_period_and_val;
} _IR_lvalue_period_and_val;

struct _IR_S_lvalue_index
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_lvalue_index _IR_S_lvalue_index;
} _IR_lvalue_index;

struct _IR_S_lvalue_index_and_val
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_lvalue_index_and_val _IR_S_lvalue_index_and_val;
} _IR_lvalue_index_and_val;

struct _IR_S_lvalue_key_index
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_lvalue_key_index _IR_S_lvalue_key_index;
} _IR_lvalue_key_index;

struct _IR_S_lvalue_key_index_and_val
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_lvalue_key_index_and_val _IR_S_lvalue_key_index_and_val;
} _IR_lvalue_key_index_and_val;

struct _IR_S_lvalue_no_testing_period
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_lvalue_no_testing_period _IR_S_lvalue_no_testing_period;
} _IR_lvalue_no_testing_period;

struct _IR_S_lvalue_no_testing_period_and_val
{
  struct _IR_S_binary_operation _IR_M_binary_operation;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_lvalue_no_testing_period_and_val _IR_S_lvalue_no_testing_period_and_val;
} _IR_lvalue_no_testing_period_and_val;

struct _IR_S_logical_or_end
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_logical_or_end _IR_S_logical_or_end;
} _IR_logical_or_end;

struct _IR_S_logical_and_end
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_logical_and_end _IR_S_logical_and_end;
} _IR_logical_and_end;

struct _IR_S_cond_end
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_cond_end _IR_S_cond_end;
} _IR_cond_end;

struct _IR_S_par_assign_end
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_par_assign_end _IR_S_par_assign_end;
} _IR_par_assign_end;

struct _IR_S_par_assign_test
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
  IR_node_t  skip_par_assign_path_pc;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_par_assign_test _IR_S_par_assign_test;
} _IR_par_assign_test;

struct _IR_S_nil
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_nil _IR_S_nil;
} _IR_nil;

struct _IR_S_friend
{
  struct _IR_S_ir_node _IR_M_ir_node;
  IR_node_t  friend_decl;
  IR_node_t  next_friend;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_friend _IR_S_friend;
} _IR_friend;

struct _IR_S_occurrence
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
  IR_node_t  decl;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_occurrence _IR_S_occurrence;
} _IR_occurrence;

struct _IR_S_external_var_occurrence
{
  struct _IR_S_occurrence _IR_M_occurrence;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_external_var_occurrence _IR_S_external_var_occurrence;
} _IR_external_var_occurrence;

struct _IR_S_local_var_occurrence
{
  struct _IR_S_occurrence _IR_M_occurrence;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_local_var_occurrence _IR_S_local_var_occurrence;
} _IR_local_var_occurrence;

struct _IR_S_var_occurrence
{
  struct _IR_S_occurrence _IR_M_occurrence;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_var_occurrence _IR_S_var_occurrence;
} _IR_var_occurrence;

struct _IR_S_lvalue_external_var_occurrence
{
  struct _IR_S_occurrence _IR_M_occurrence;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_lvalue_external_var_occurrence _IR_S_lvalue_external_var_occurrence;
} _IR_lvalue_external_var_occurrence;

struct _IR_S_lvalue_external_var_occurrence_and_val
{
  struct _IR_S_occurrence _IR_M_occurrence;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_lvalue_external_var_occurrence_and_val _IR_S_lvalue_external_var_occurrence_and_val;
} _IR_lvalue_external_var_occurrence_and_val;

struct _IR_S_local_lvalue_var_occurrence
{
  struct _IR_S_occurrence _IR_M_occurrence;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_local_lvalue_var_occurrence _IR_S_local_lvalue_var_occurrence;
} _IR_local_lvalue_var_occurrence;

struct _IR_S_local_lvalue_var_occurrence_and_val
{
  struct _IR_S_occurrence _IR_M_occurrence;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_local_lvalue_var_occurrence_and_val _IR_S_local_lvalue_var_occurrence_and_val;
} _IR_local_lvalue_var_occurrence_and_val;

struct _IR_S_lvalue_var_occurrence
{
  struct _IR_S_occurrence _IR_M_occurrence;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_lvalue_var_occurrence _IR_S_lvalue_var_occurrence;
} _IR_lvalue_var_occurrence;

struct _IR_S_lvalue_var_occurrence_and_val
{
  struct _IR_S_occurrence _IR_M_occurrence;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_lvalue_var_occurrence_and_val _IR_S_lvalue_var_occurrence_and_val;
} _IR_lvalue_var_occurrence_and_val;

struct _IR_S_external_func_occurrence
{
  struct _IR_S_occurrence _IR_M_occurrence;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_external_func_occurrence _IR_S_external_func_occurrence;
} _IR_external_func_occurrence;

struct _IR_S_func_occurrence
{
  struct _IR_S_occurrence _IR_M_occurrence;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_func_occurrence _IR_S_func_occurrence;
} _IR_func_occurrence;

struct _IR_S_class_occurrence
{
  struct _IR_S_occurrence _IR_M_occurrence;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_class_occurrence _IR_S_class_occurrence;
} _IR_class_occurrence;

struct _IR_S_if_finish
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_if_finish _IR_S_if_finish;
} _IR_if_finish;

struct _IR_S_for_finish
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_for_finish _IR_S_for_finish;
} _IR_for_finish;

struct _IR_S_wait_finish
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_wait_finish _IR_S_wait_finish;
} _IR_wait_finish;

struct _IR_S_foreach_start
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_foreach_start _IR_S_foreach_start;
} _IR_foreach_start;

struct _IR_S_foreach_next_iteration
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_foreach_next_iteration _IR_S_foreach_next_iteration;
} _IR_foreach_next_iteration;

struct _IR_S_block_finish
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
  IR_node_t  block;
  bool_t  simple_block_finish_flag;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_block_finish _IR_S_block_finish;
} _IR_block_finish;

struct _IR_S_catches_finish
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_catches_finish _IR_S_catches_finish;
} _IR_catches_finish;

struct _IR_S_pop_func_call
{
  struct _IR_S_generic_pos _IR_M_generic_pos;
};

typedef struct
{
  IR_node_mode_t _IR_node_mode;
  struct _IR_S_pop_func_call _IR_S_pop_func_call;
} _IR_pop_func_call;

typedef struct IR_node
{
  IR_node_mode_t _IR_node_mode;
} _IR__root;

typedef struct
{
  IR_node_mode_t _IR_node_mode;
} _IR__error;

#define IR_NODE_MODE(t) ((t)->_IR_node_mode)

extern char *IR_node_name[];

extern unsigned char IR_node_size[];

#define IR_NODE_LEVEL(node) _IR_node_level [IR_NODE_MODE (node)]

#define IR_IS_TYPE(type, super) ((_IR_is_type [super] [type /8] >> (type % 8)) & 1)

#define IR_IS_OF_TYPE(node, super)IR_IS_TYPE (IR_NODE_MODE (node), super)

extern void _IR_set_double_field_value
  (IR_double_link_t link, IR_node_t value, int disp, int flag);

extern IR_double_link_t IR__first_double_link (IR_node_t node);

#define IR__next_double_link(prev_double_link) ((prev_double_link)->next_link)

#define IR__previous_double_link(next_double_link) ((next_double_link)->previous_link)

#define IR__owner(link) ((link)->link_owner)

#define IR_next_pc(_node) (((_IR_ir_node *) (_node))->_IR_S_ir_node.next_pc)

#define IR_char_value(_node) (((_IR_unique_char *) (_node))->_IR_S_unique_char.char_value)

#define IR_pos(_node) (((_IR_generic_pos *) (_node))->_IR_S_generic_pos.pos)

#define IR_unique_char(_node) (((_IR_char *) (_node))->_IR_S_char.unique_char)

#define IR_ch_val(_node) (((_IR_char *) (_node))->_IR_S_char.ch_val)

#define IR_int_value(_node) (((_IR_unique_int *) (_node))->_IR_S_unique_int.int_value)

#define IR_unique_int(_node) (((_IR_int *) (_node))->_IR_S_int.unique_int)

#define IR_i_val(_node) (((_IR_int *) (_node))->_IR_S_int.i_val)

#define IR_float_value(_node) (((_IR_unique_float *) (_node))->_IR_S_unique_float.float_value)

#define IR_unique_float(_node) (((_IR_float *) (_node))->_IR_S_float.unique_float)

#define IR_f_val(_node) (((_IR_float *) (_node))->_IR_S_float.f_val)

#define IR_string_value(_node) (((_IR_unique_string *) (_node))->_IR_S_unique_string.string_value)

#define IR_unique_string(_node) (((_IR_string *) (_node))->_IR_S_string.unique_string)

#define IR_str_val(_node) (((_IR_string *) (_node))->_IR_S_string.str_val)

#define IR_ident_string(_node) (((_IR_unique_ident *) (_node))->_IR_S_unique_ident.ident_string)

#define IR_block_decl_ident_number(_node) (((_IR_unique_ident *) (_node))->_IR_S_unique_ident.block_decl_ident_number)

#define IR_it_is_declared_in_block(_node) (((_IR_unique_ident *) (_node))->_IR_S_unique_ident.it_is_declared_in_block)

#define IR_unique_ident(_node) (((_IR_ident *) (_node))->_IR_S_ident.unique_ident)

#define IR_value_type(_node) (((_IR_operation *) (_node))->_IR_S_operation.value_type)

#define IR_left_operand(_node) (((_IR_binary_operation *) (_node))->_IR_S_binary_operation.left_operand)

#define IR_right_operand(_node) (((_IR_binary_operation *) (_node))->_IR_S_binary_operation.right_operand)

#define IR_operand(_node) (((_IR_unary_operation *) (_node))->_IR_S_unary_operation.operand)

#define IR_right_block_decl_ident_number(_node) (((_IR_member_access *) (_node))->_IR_S_member_access.right_block_decl_ident_number)

#define IR_short_path_pc(_node) (((_IR_logical_binary_operation *) (_node))->_IR_S_logical_binary_operation.short_path_pc)

#define IR_cond(_node) (((_IR_cond *) (_node))->_IR_S_cond.cond)

#define IR_false_path_pc(_node) (((_IR_cond *) (_node))->_IR_S_cond.false_path_pc)

#define IR_format(_node) (((_IR_format_vectorof *) (_node))->_IR_S_format_vectorof.format)

#define IR_elist(_node) (((_IR_vector_table *) (_node))->_IR_S_vector_table.elist)

#define IR_parts_number(_node) (((_IR_vector_table *) (_node))->_IR_S_vector_table.parts_number)

#define IR_repetition_key(_node) (((_IR_elist_element *) (_node))->_IR_S_elist_element.repetition_key)

#define IR_expr(_node) (((_IR_elist_element *) (_node))->_IR_S_elist_element.expr)

#define IR_next_elist(_node) (((_IR_elist_element *) (_node))->_IR_S_elist_element.next_elist)

#define IR_func_expr(_node) (((_IR_class_func_thread_call *) (_node))->_IR_S_class_func_thread_call.func_expr)

#define IR_actuals(_node) (((_IR_class_func_thread_call *) (_node))->_IR_S_class_func_thread_call.actuals)

#define IR_class_func_thread_call_parameters_number(_node) (((_IR_class_func_thread_call *) (_node))->_IR_S_class_func_thread_call.class_func_thread_call_parameters_number)

#define IR_ident(_node) (*(IR_node_t *) ((char *) (_node) + _IR_D_ident [IR_NODE_MODE ((_node))]))

#define IR_access_ident_public_flag(_node) (((_IR_access_ident *) (_node))->_IR_S_access_ident.access_ident_public_flag)

#define IR_friend_flag(_node) (((_IR_access_ident *) (_node))->_IR_S_access_ident.friend_flag)

#define IR_next_access_ident(_node) (((_IR_access_ident *) (_node))->_IR_S_access_ident.next_access_ident)

#define IR_next_stmt(_node) (((_IR_stmt *) (_node))->_IR_S_stmt.next_stmt)

#define IR_assignment_var(_node) (((_IR_assign_stmt *) (_node))->_IR_S_assign_stmt.assignment_var)

#define IR_assignment_expr(_node) (((_IR_assign_stmt *) (_node))->_IR_S_assign_stmt.assignment_expr)

#define IR_swap_var1(_node) (((_IR_swap *) (_node))->_IR_S_swap.swap_var1)

#define IR_swap_var2(_node) (((_IR_swap *) (_node))->_IR_S_swap.swap_var2)

#define IR_proc_expr(_node) (((_IR_procedure_call *) (_node))->_IR_S_procedure_call.proc_expr)

#define IR_proc_actuals(_node) (((_IR_procedure_call *) (_node))->_IR_S_procedure_call.proc_actuals)

#define IR_procedure_call_pars_number(_node) (((_IR_procedure_call *) (_node))->_IR_S_procedure_call.procedure_call_pars_number)

#define IR_if_expr(_node) (((_IR_if_stmt *) (_node))->_IR_S_if_stmt.if_expr)

#define IR_if_part(_node) (((_IR_if_stmt *) (_node))->_IR_S_if_stmt.if_part)

#define IR_else_part(_node) (((_IR_if_stmt *) (_node))->_IR_S_if_stmt.else_part)

#define IR_else_part_pc(_node) (((_IR_if_stmt *) (_node))->_IR_S_if_stmt.else_part_pc)

#define IR_for_initial_stmt(_node) (((_IR_for_stmt *) (_node))->_IR_S_for_stmt.for_initial_stmt)

#define IR_for_guard_expr(_node) (((_IR_for_stmt *) (_node))->_IR_S_for_stmt.for_guard_expr)

#define IR_for_iterate_stmt(_node) (((_IR_for_stmt *) (_node))->_IR_S_for_stmt.for_iterate_stmt)

#define IR_for_stmts(_node) (((_IR_for_stmt *) (_node))->_IR_S_for_stmt.for_stmts)

#define IR_for_body_pc(_node) (((_IR_for_stmt *) (_node))->_IR_S_for_stmt.for_body_pc)

#define IR_foreach_designator(_node) (((_IR_foreach_stmt *) (_node))->_IR_S_foreach_stmt.foreach_designator)

#define IR_foreach_table(_node) (((_IR_foreach_stmt *) (_node))->_IR_S_foreach_stmt.foreach_table)

#define IR_foreach_stmts(_node) (((_IR_foreach_stmt *) (_node))->_IR_S_foreach_stmt.foreach_stmts)

#define IR_foreach_body_pc(_node) (((_IR_foreach_stmt *) (_node))->_IR_S_foreach_stmt.foreach_body_pc)

#define IR_number_of_surrounding_blocks(_node) (((_IR_for_break_contintue *) (_node))->_IR_S_for_break_contintue.number_of_surrounding_blocks)

#define IR_returned_expr(_node) (((_IR_return_with_result *) (_node))->_IR_S_return_with_result.returned_expr)

#define IR_throw_expr(_node) (((_IR_throw *) (_node))->_IR_S_throw.throw_expr)

#define IR_wait_guard_expr(_node) (((_IR_wait_stmt *) (_node))->_IR_S_wait_stmt.wait_guard_expr)

#define IR_wait_stmt(_node) (((_IR_wait_stmt *) (_node))->_IR_S_wait_stmt.wait_stmt)

#define IR_start_wait_guard_expr_pc(_node) (((_IR_wait_stmt *) (_node))->_IR_S_wait_stmt.start_wait_guard_expr_pc)

#define IR_block_scope(_node) (((_IR_block *) (_node))->_IR_S_block.block_scope)

#define IR_block_stmts(_node) (((_IR_block *) (_node))->_IR_S_block.block_stmts)

#define IR_func_class_ext(_node) (((_IR_block *) (_node))->_IR_S_block.func_class_ext)

#define IR_access_list(_node) (((_IR_block *) (_node))->_IR_S_block.access_list)

#define IR_exceptions(_node) (((_IR_block *) (_node))->_IR_S_block.exceptions)

#define IR_block_level(_node) (((_IR_block *) (_node))->_IR_S_block.block_level)

#define IR_decls_number(_node) (((_IR_block *) (_node))->_IR_S_block.decls_number)

#define IR_vars_number(_node) (((_IR_block *) (_node))->_IR_S_block.vars_number)

#define IR_temporary_vars_number(_node) (((_IR_block *) (_node))->_IR_S_block.temporary_vars_number)

#define IR_block_number(_node) (((_IR_block *) (_node))->_IR_S_block.block_number)

#define IR_catch_list_pc(_node) (*(IR_node_t *) ((char *) (_node) + _IR_D_catch_list_pc [IR_NODE_MODE ((_node))]))

#define IR_simple_block_flag(_node) (((_IR_block *) (_node))->_IR_S_block.simple_block_flag)

#define IR_extended_life_context_flag(_node) (((_IR_block *) (_node))->_IR_S_block.extended_life_context_flag)

#define IR_friends(_node) (((_IR_block *) (_node))->_IR_S_block.friends)

#define IR_cached_container_address(_node) (((_IR_block *) (_node))->_IR_S_block.cached_container_address)

#define IR_cached_container_tick(_node) (((_IR_block *) (_node))->_IR_S_block.cached_container_tick)

#define IR_block_saved_sync_flag(_node) (((_IR_block *) (_node))->_IR_S_block.block_saved_sync_flag)

#define IR_exception_class_expr(_node) (((_IR_exception *) (_node))->_IR_S_exception.exception_class_expr)

#define IR_catch_block(_node) (((_IR_exception *) (_node))->_IR_S_exception.catch_block)

#define IR_next_exception(_node) (((_IR_exception *) (_node))->_IR_S_exception.next_exception)

#define IR_scope(_node) (((_IR_decl *) (_node))->_IR_S_decl.scope)

#define IR_implementation_func(_node) (((_IR_decl *) (_node))->_IR_S_decl.implementation_func)

#define IR_public_flag(_node) (((_IR_decl *) (_node))->_IR_S_decl.public_flag)

#define IR_access_ident(_node) (((_IR_decl *) (_node))->_IR_S_decl.access_ident)

#define IR_const_flag(_node) (((_IR_may_be_const_decl *) (_node))->_IR_S_may_be_const_decl.const_flag)

#define IR_var_number_in_block(_node) (((_IR_var *) (_node))->_IR_S_var.var_number_in_block)

#define IR_no(_node) (*(int_t *) ((char *) (_node) + _IR_D_no [IR_NODE_MODE ((_node))]))

#define IR_parameters_number(_node) (((_IR_func_or_class *) (_node))->_IR_S_func_or_class.parameters_number)

#define IR_args_flag(_node) (((_IR_func_or_class *) (_node))->_IR_S_func_or_class.args_flag)

#define IR_final_flag(_node) (((_IR_func_or_class *) (_node))->_IR_S_func_or_class.final_flag)

#define IR_calls_number(_node) (((_IR_func_or_class *) (_node))->_IR_S_func_or_class.calls_number)

#define IR_exec_time(_node) (((_IR_func_or_class *) (_node))->_IR_S_func_or_class.exec_time)

#define IR_interrupts_number(_node) (((_IR_func_or_class *) (_node))->_IR_S_func_or_class.interrupts_number)

#define IR_thread_flag(_node) (((_IR_func *) (_node))->_IR_S_func.thread_flag)

#define IR_simple_class_flag(_node) (((_IR_class *) (_node))->_IR_S_class.simple_class_flag)

#define IR_skip_par_assign_path_pc(_node) (((_IR_par_assign_test *) (_node))->_IR_S_par_assign_test.skip_par_assign_path_pc)

#define IR_friend_decl(_node) (((_IR_friend *) (_node))->_IR_S_friend.friend_decl)

#define IR_next_friend(_node) (((_IR_friend *) (_node))->_IR_S_friend.next_friend)

#define IR_decl_number_in_block(_node) (*(integer_t *) ((char *) (_node) + _IR_D_decl_number_in_block [IR_NODE_MODE ((_node))]))

#define IR_decl(_node) (((_IR_occurrence *) (_node))->_IR_S_occurrence.decl)

#define IR_block(_node) (((_IR_block_finish *) (_node))->_IR_S_block_finish.block)

#define IR_simple_block_finish_flag(_node) (((_IR_block_finish *) (_node))->_IR_S_block_finish.simple_block_finish_flag)

#define IR_address(_node) (*(string_t *) ((char *) (_node) + _IR_D_address [IR_NODE_MODE ((_node))]))

extern void IR__set_double_link (IR_double_link_t link, IR_node_t value);

#define IR_set_next_pc(_node, _value) ((((_IR_ir_node *) (_node))->_IR_S_ir_node.next_pc) = (_value))

#define IR_set_char_value(_node, _value) ((((_IR_unique_char *) (_node))->_IR_S_unique_char.char_value) = (_value))

#define IR_set_pos(_node, _value) ((((_IR_generic_pos *) (_node))->_IR_S_generic_pos.pos) = (_value))

#define IR_set_unique_char(_node, _value) ((((_IR_char *) (_node))->_IR_S_char.unique_char) = (_value))

#define IR_set_ch_val(_node, _value) ((((_IR_char *) (_node))->_IR_S_char.ch_val) = (_value))

#define IR_set_int_value(_node, _value) ((((_IR_unique_int *) (_node))->_IR_S_unique_int.int_value) = (_value))

#define IR_set_unique_int(_node, _value) ((((_IR_int *) (_node))->_IR_S_int.unique_int) = (_value))

#define IR_set_i_val(_node, _value) ((((_IR_int *) (_node))->_IR_S_int.i_val) = (_value))

#define IR_set_float_value(_node, _value) ((((_IR_unique_float *) (_node))->_IR_S_unique_float.float_value) = (_value))

#define IR_set_unique_float(_node, _value) ((((_IR_float *) (_node))->_IR_S_float.unique_float) = (_value))

#define IR_set_f_val(_node, _value) ((((_IR_float *) (_node))->_IR_S_float.f_val) = (_value))

#define IR_set_string_value(_node, _value) ((((_IR_unique_string *) (_node))->_IR_S_unique_string.string_value) = (_value))

#define IR_set_unique_string(_node, _value) ((((_IR_string *) (_node))->_IR_S_string.unique_string) = (_value))

#define IR_set_str_val(_node, _value) ((((_IR_string *) (_node))->_IR_S_string.str_val) = (_value))

#define IR_set_ident_string(_node, _value) ((((_IR_unique_ident *) (_node))->_IR_S_unique_ident.ident_string) = (_value))

#define IR_set_block_decl_ident_number(_node, _value) ((((_IR_unique_ident *) (_node))->_IR_S_unique_ident.block_decl_ident_number) = (_value))

#define IR_set_it_is_declared_in_block(_node, _value) ((((_IR_unique_ident *) (_node))->_IR_S_unique_ident.it_is_declared_in_block) = (_value))

#define IR_set_unique_ident(_node, _value) ((((_IR_ident *) (_node))->_IR_S_ident.unique_ident) = (_value))

#define IR_set_value_type(_node, _value) ((((_IR_operation *) (_node))->_IR_S_operation.value_type) = (_value))

#define IR_set_left_operand(_node, _value) ((((_IR_binary_operation *) (_node))->_IR_S_binary_operation.left_operand) = (_value))

#define IR_set_right_operand(_node, _value) ((((_IR_binary_operation *) (_node))->_IR_S_binary_operation.right_operand) = (_value))

#define IR_set_operand(_node, _value) ((((_IR_unary_operation *) (_node))->_IR_S_unary_operation.operand) = (_value))

#define IR_set_right_block_decl_ident_number(_node, _value) ((((_IR_member_access *) (_node))->_IR_S_member_access.right_block_decl_ident_number) = (_value))

#define IR_set_short_path_pc(_node, _value) ((((_IR_logical_binary_operation *) (_node))->_IR_S_logical_binary_operation.short_path_pc) = (_value))

#define IR_set_cond(_node, _value) ((((_IR_cond *) (_node))->_IR_S_cond.cond) = (_value))

#define IR_set_false_path_pc(_node, _value) ((((_IR_cond *) (_node))->_IR_S_cond.false_path_pc) = (_value))

#define IR_set_format(_node, _value) ((((_IR_format_vectorof *) (_node))->_IR_S_format_vectorof.format) = (_value))

#define IR_set_elist(_node, _value) ((((_IR_vector_table *) (_node))->_IR_S_vector_table.elist) = (_value))

#define IR_set_parts_number(_node, _value) ((((_IR_vector_table *) (_node))->_IR_S_vector_table.parts_number) = (_value))

#define IR_set_repetition_key(_node, _value) ((((_IR_elist_element *) (_node))->_IR_S_elist_element.repetition_key) = (_value))

#define IR_set_expr(_node, _value) ((((_IR_elist_element *) (_node))->_IR_S_elist_element.expr) = (_value))

#define IR_set_next_elist(_node, _value) ((((_IR_elist_element *) (_node))->_IR_S_elist_element.next_elist) = (_value))

#define IR_set_func_expr(_node, _value) ((((_IR_class_func_thread_call *) (_node))->_IR_S_class_func_thread_call.func_expr) = (_value))

#define IR_set_actuals(_node, _value) ((((_IR_class_func_thread_call *) (_node))->_IR_S_class_func_thread_call.actuals) = (_value))

#define IR_set_class_func_thread_call_parameters_number(_node, _value) ((((_IR_class_func_thread_call *) (_node))->_IR_S_class_func_thread_call.class_func_thread_call_parameters_number) = (_value))

#define IR_set_ident(_node, _value) ((*(IR_node_t *) ((char *) (_node) + _IR_D_ident [IR_NODE_MODE ((_node))])) = (_value))

#define IR_set_access_ident_public_flag(_node, _value) ((((_IR_access_ident *) (_node))->_IR_S_access_ident.access_ident_public_flag) = (_value))

#define IR_set_friend_flag(_node, _value) ((((_IR_access_ident *) (_node))->_IR_S_access_ident.friend_flag) = (_value))

#define IR_set_next_access_ident(_node, _value) ((((_IR_access_ident *) (_node))->_IR_S_access_ident.next_access_ident) = (_value))

#define IR_set_next_stmt(_node, _value) ((((_IR_stmt *) (_node))->_IR_S_stmt.next_stmt) = (_value))

#define IR_set_assignment_var(_node, _value) ((((_IR_assign_stmt *) (_node))->_IR_S_assign_stmt.assignment_var) = (_value))

#define IR_set_assignment_expr(_node, _value) ((((_IR_assign_stmt *) (_node))->_IR_S_assign_stmt.assignment_expr) = (_value))

#define IR_set_swap_var1(_node, _value) ((((_IR_swap *) (_node))->_IR_S_swap.swap_var1) = (_value))

#define IR_set_swap_var2(_node, _value) ((((_IR_swap *) (_node))->_IR_S_swap.swap_var2) = (_value))

#define IR_set_proc_expr(_node, _value) ((((_IR_procedure_call *) (_node))->_IR_S_procedure_call.proc_expr) = (_value))

#define IR_set_proc_actuals(_node, _value) ((((_IR_procedure_call *) (_node))->_IR_S_procedure_call.proc_actuals) = (_value))

#define IR_set_procedure_call_pars_number(_node, _value) ((((_IR_procedure_call *) (_node))->_IR_S_procedure_call.procedure_call_pars_number) = (_value))

#define IR_set_if_expr(_node, _value) ((((_IR_if_stmt *) (_node))->_IR_S_if_stmt.if_expr) = (_value))

#define IR_set_if_part(_node, _value) ((((_IR_if_stmt *) (_node))->_IR_S_if_stmt.if_part) = (_value))

#define IR_set_else_part(_node, _value) ((((_IR_if_stmt *) (_node))->_IR_S_if_stmt.else_part) = (_value))

#define IR_set_else_part_pc(_node, _value) ((((_IR_if_stmt *) (_node))->_IR_S_if_stmt.else_part_pc) = (_value))

#define IR_set_for_initial_stmt(_node, _value) ((((_IR_for_stmt *) (_node))->_IR_S_for_stmt.for_initial_stmt) = (_value))

#define IR_set_for_guard_expr(_node, _value) ((((_IR_for_stmt *) (_node))->_IR_S_for_stmt.for_guard_expr) = (_value))

#define IR_set_for_iterate_stmt(_node, _value) ((((_IR_for_stmt *) (_node))->_IR_S_for_stmt.for_iterate_stmt) = (_value))

#define IR_set_for_stmts(_node, _value) ((((_IR_for_stmt *) (_node))->_IR_S_for_stmt.for_stmts) = (_value))

#define IR_set_for_body_pc(_node, _value) ((((_IR_for_stmt *) (_node))->_IR_S_for_stmt.for_body_pc) = (_value))

#define IR_set_foreach_designator(_node, _value) ((((_IR_foreach_stmt *) (_node))->_IR_S_foreach_stmt.foreach_designator) = (_value))

#define IR_set_foreach_table(_node, _value) ((((_IR_foreach_stmt *) (_node))->_IR_S_foreach_stmt.foreach_table) = (_value))

#define IR_set_foreach_stmts(_node, _value) ((((_IR_foreach_stmt *) (_node))->_IR_S_foreach_stmt.foreach_stmts) = (_value))

#define IR_set_foreach_body_pc(_node, _value) ((((_IR_foreach_stmt *) (_node))->_IR_S_foreach_stmt.foreach_body_pc) = (_value))

#define IR_set_number_of_surrounding_blocks(_node, _value) ((((_IR_for_break_contintue *) (_node))->_IR_S_for_break_contintue.number_of_surrounding_blocks) = (_value))

#define IR_set_returned_expr(_node, _value) ((((_IR_return_with_result *) (_node))->_IR_S_return_with_result.returned_expr) = (_value))

#define IR_set_throw_expr(_node, _value) ((((_IR_throw *) (_node))->_IR_S_throw.throw_expr) = (_value))

#define IR_set_wait_guard_expr(_node, _value) ((((_IR_wait_stmt *) (_node))->_IR_S_wait_stmt.wait_guard_expr) = (_value))

#define IR_set_wait_stmt(_node, _value) ((((_IR_wait_stmt *) (_node))->_IR_S_wait_stmt.wait_stmt) = (_value))

#define IR_set_start_wait_guard_expr_pc(_node, _value) ((((_IR_wait_stmt *) (_node))->_IR_S_wait_stmt.start_wait_guard_expr_pc) = (_value))

#define IR_set_block_scope(_node, _value) ((((_IR_block *) (_node))->_IR_S_block.block_scope) = (_value))

#define IR_set_block_stmts(_node, _value) ((((_IR_block *) (_node))->_IR_S_block.block_stmts) = (_value))

#define IR_set_func_class_ext(_node, _value) ((((_IR_block *) (_node))->_IR_S_block.func_class_ext) = (_value))

#define IR_set_access_list(_node, _value) ((((_IR_block *) (_node))->_IR_S_block.access_list) = (_value))

#define IR_set_exceptions(_node, _value) ((((_IR_block *) (_node))->_IR_S_block.exceptions) = (_value))

#define IR_set_block_level(_node, _value) ((((_IR_block *) (_node))->_IR_S_block.block_level) = (_value))

#define IR_set_decls_number(_node, _value) ((((_IR_block *) (_node))->_IR_S_block.decls_number) = (_value))

#define IR_set_vars_number(_node, _value) ((((_IR_block *) (_node))->_IR_S_block.vars_number) = (_value))

#define IR_set_temporary_vars_number(_node, _value) ((((_IR_block *) (_node))->_IR_S_block.temporary_vars_number) = (_value))

#define IR_set_block_number(_node, _value) ((((_IR_block *) (_node))->_IR_S_block.block_number) = (_value))

#define IR_set_catch_list_pc(_node, _value) ((*(IR_node_t *) ((char *) (_node) + _IR_D_catch_list_pc [IR_NODE_MODE ((_node))])) = (_value))

#define IR_set_simple_block_flag(_node, _value) ((((_IR_block *) (_node))->_IR_S_block.simple_block_flag) = (_value))

#define IR_set_extended_life_context_flag(_node, _value) ((((_IR_block *) (_node))->_IR_S_block.extended_life_context_flag) = (_value))

#define IR_set_friends(_node, _value) ((((_IR_block *) (_node))->_IR_S_block.friends) = (_value))

#define IR_set_cached_container_address(_node, _value) ((((_IR_block *) (_node))->_IR_S_block.cached_container_address) = (_value))

#define IR_set_cached_container_tick(_node, _value) ((((_IR_block *) (_node))->_IR_S_block.cached_container_tick) = (_value))

#define IR_set_block_saved_sync_flag(_node, _value) ((((_IR_block *) (_node))->_IR_S_block.block_saved_sync_flag) = (_value))

#define IR_set_exception_class_expr(_node, _value) ((((_IR_exception *) (_node))->_IR_S_exception.exception_class_expr) = (_value))

#define IR_set_catch_block(_node, _value) ((((_IR_exception *) (_node))->_IR_S_exception.catch_block) = (_value))

#define IR_set_next_exception(_node, _value) ((((_IR_exception *) (_node))->_IR_S_exception.next_exception) = (_value))

#define IR_set_scope(_node, _value) ((((_IR_decl *) (_node))->_IR_S_decl.scope) = (_value))

#define IR_set_implementation_func(_node, _value) ((((_IR_decl *) (_node))->_IR_S_decl.implementation_func) = (_value))

#define IR_set_public_flag(_node, _value) ((((_IR_decl *) (_node))->_IR_S_decl.public_flag) = (_value))

#define IR_set_access_ident(_node, _value) ((((_IR_decl *) (_node))->_IR_S_decl.access_ident) = (_value))

#define IR_set_const_flag(_node, _value) ((((_IR_may_be_const_decl *) (_node))->_IR_S_may_be_const_decl.const_flag) = (_value))

#define IR_set_var_number_in_block(_node, _value) ((((_IR_var *) (_node))->_IR_S_var.var_number_in_block) = (_value))

#define IR_set_no(_node, _value) ((*(int_t *) ((char *) (_node) + _IR_D_no [IR_NODE_MODE ((_node))])) = (_value))

#define IR_set_parameters_number(_node, _value) ((((_IR_func_or_class *) (_node))->_IR_S_func_or_class.parameters_number) = (_value))

#define IR_set_args_flag(_node, _value) ((((_IR_func_or_class *) (_node))->_IR_S_func_or_class.args_flag) = (_value))

#define IR_set_final_flag(_node, _value) ((((_IR_func_or_class *) (_node))->_IR_S_func_or_class.final_flag) = (_value))

#define IR_set_calls_number(_node, _value) ((((_IR_func_or_class *) (_node))->_IR_S_func_or_class.calls_number) = (_value))

#define IR_set_exec_time(_node, _value) ((((_IR_func_or_class *) (_node))->_IR_S_func_or_class.exec_time) = (_value))

#define IR_set_interrupts_number(_node, _value) ((((_IR_func_or_class *) (_node))->_IR_S_func_or_class.interrupts_number) = (_value))

#define IR_set_thread_flag(_node, _value) ((((_IR_func *) (_node))->_IR_S_func.thread_flag) = (_value))

#define IR_set_simple_class_flag(_node, _value) ((((_IR_class *) (_node))->_IR_S_class.simple_class_flag) = (_value))

#define IR_set_skip_par_assign_path_pc(_node, _value) ((((_IR_par_assign_test *) (_node))->_IR_S_par_assign_test.skip_par_assign_path_pc) = (_value))

#define IR_set_friend_decl(_node, _value) ((((_IR_friend *) (_node))->_IR_S_friend.friend_decl) = (_value))

#define IR_set_next_friend(_node, _value) ((((_IR_friend *) (_node))->_IR_S_friend.next_friend) = (_value))

#define IR_set_decl_number_in_block(_node, _value) ((*(integer_t *) ((char *) (_node) + _IR_D_decl_number_in_block [IR_NODE_MODE ((_node))])) = (_value))

#define IR_set_decl(_node, _value) ((((_IR_occurrence *) (_node))->_IR_S_occurrence.decl) = (_value))

#define IR_set_block(_node, _value) ((((_IR_block_finish *) (_node))->_IR_S_block_finish.block) = (_value))

#define IR_set_simple_block_finish_flag(_node, _value) ((((_IR_block_finish *) (_node))->_IR_S_block_finish.simple_block_finish_flag) = (_value))

#define IR_set_address(_node, _value) ((*(string_t *) ((char *) (_node) + _IR_D_address [IR_NODE_MODE ((_node))])) = (_value))

extern IR_node_t IR_create_node (IR_node_mode_t node_mode);

extern void IR_start (void);

extern void IR_stop (void);


#line 84 "./d_ir.sprut"



/* These macros for storage management of DINO internal representation: */

/* Start work with the storage manager -- see DINO documentation. */

#define IR_START_ALLOC()    OS_CREATE (irp, 0)

/* Finish work with the storage manager -- see DINO documentation. */

#define IR_STOP_ALLOC()     OS_DELETE (irp)

/* Allocate storage for internal representation of given size
   -- see DINO documentation. */

#define IR_ALLOC(ptr, size, ptr_type)\
  do {\
    OS_TOP_EXPAND (irp, size); ptr = (ptr_type) OS_TOP_BEGIN (irp);\
    OS_TOP_FINISH (irp);\
  } while (0);

/* Free storage of internal representation of given size -- see DINO
   documentation. */

#define IR_FREE(ptr, size)


/* These macros are analogous to ones of package `object-stack'
   worked with storage of DINO internal representation: */

/* Start new internal representation object -- see also package
   `object-stack'. */

#define IR_TOP_FINISH()  OS_TOP_FINISH (irp)

/* Nullify current internal representation object -- see also package
   `object-stack'. */

#define IR_TOP_NULLIFY()  OS_TOP_NULLIFY (irp)

/* Shorten current internal representation object on given number bytes -- see
   also package `object-stack'. */

#define IR_TOP_SHORTEN(length) OS_TOP_SHORTEN (irp, length)

/* Return start address of current internal representation object -- see also
   package `object-stack'. */

#define IR_TOP_BEGIN()  OS_TOP_BEGIN (irp)

/* Return length of current internal representation object in bytes -- see
   also package `object-stack'. */

#define IR_TOP_LENGTH()  OS_TOP_LENGTH (irp)

/* Expand current internal representation object -- see also package
   `object-stack'. */

#define IR_TOP_EXPAND(length)  OS_TOP_EXPAND (irp, length)

/* Add byte to the end of current internal representation object -- see also
   package `object-stack'. */

#define IR_TOP_ADD_BYTE(b)  OS_TOP_ADD_BYTE (irp, b)

/* Add string to the end of current internal representation object -- see also
   package `object-stack'. */

#define IR_TOP_ADD_STRING(str)  OS_TOP_ADD_STRING (irp, str)

/* Add memory of given length to the end of current internal representation
   object -- see also package `object-stack'. */

#define IR_TOP_ADD_MEMORY(mem, length)  OS_TOP_ADD_MEMORY (irp, mem, length)

extern os_t irp;


extern void initiate_table (void);
extern void delete_table (void);
extern IR_node_t *find_table_entry (IR_node_t ref, int reserve);
extern void include_to_table (IR_node_t ref);
extern IR_node_t find_unique_ident (char *string);
extern IR_node_t create_node (IR_node_mode_t mode);
extern IR_node_t create_node_with_pos (IR_node_mode_t mode, position_t pos);
extern IR_node_t create_unique_node_with_string (IR_node_mode_t m,
                                                 const char *str,
                                                 char **string_in_pool_ptr);
extern IR_node_t create_unique_ident_node (const char *str);
extern IR_node_mode_t occurrence_mode (IR_node_t decl);
extern IR_node_t create_occurrence_node (IR_node_t decl, position_t pos, int);
extern void initiate_icode (void);

extern IR_node_t find_decl_in_given_scope (IR_node_t ident, IR_node_t scope);
extern IR_node_t find_decl (IR_node_t ident, IR_node_t scope);
extern void set_environment_declarations (IR_node_t environment_scope);

extern IR_node_t get_ident_node (char *name, position_t pos);
extern IR_node_t get_int_node (int_t number_value, position_t pos);
extern IR_node_t get_float_node (floating_t number_value, position_t pos);

/* Variable name for the rest parameter values (...). */
#define ARGS_NAME "args"

/* This is name of varibale which will contains catdhed exception in
   corresponding catch block. */

#define CATCH_EXCEPTION_NAME "e"

extern IR_node_t temp_unique_ident;
extern IR_node_t temp_unique_int;
extern IR_node_t temp_unique_float;
extern IR_node_t temp_unique_char;
extern IR_node_t temp_unique_string;
extern IR_node_t temp_ident;
extern IR_node_t temp_decl;
extern IR_node_t temp_var_occurrence;
extern IR_node_t destroy_unique_ident;

#define IR_POINTER(v) ((IR_node_t)(v))

/* This is type definition of program counter.  All nodes (except for
   unique nodes and var_node, func_node, class_node) is
   chained by members (one or two) of such type.  Remember that
   func may suspend the process therefore expr should be
   also be chained by program counter.  If it is needed to execute the
   following and its pc is NULL than it means that finishing current
   process is needed.  There is process for block corresponding all
   program and its finishing means that program exits. */

typedef IR_node_t pc_t;

/* Macros for transformation of given node reference to previously
   mentioned type. */

#define PC(v) ((pc_t)(v))

/* This is trick (usage that IR_NODE_MODE is macro)!!!  Therefore the
   following macro must apply to node with the same structure as node
   with mode m. */

#define IR_SET_MODE(v, m) (IR_NODE_MODE (v) = (m))

/* The macro call value is pointer to unique ident node of
   corresponding ident node V.  V may be NULL. */

#define IR_GET_UNIQUE_IDENT(v)\
  ((v) == NULL ? v : IR_unique_ident (v))

/* The following macro value is standard environment which is parsed
   before the program. The last character must be a space for correct
   parsing. */
#define ENVIRONMENT \
  "class except () { \n"\
  "  class error (msg){\n"\
  "    class signal (msg) {\n"\
  "      class sigint (msg) {}\n"\
  "      class sigill (msg) {}\n"\
  "      class sigabrt (msg) {}\n"\
  "      class sigfpe (msg) {}\n"\
  "      class sigterm (msg) {}\n"\
  "      class sigsegv (msg) {}\n"\
  "    }\n"\
  "    class invenv (msg) {}\n"\
  "    class invop (msg) {\n"\
  "      class optype (msg) {}\n"\
  "      class opvalue (msg) {}\n"\
  "    }\n"\
  "    class invindex (msg) {\n"\
  "      class indextype (msg) {}\n"\
  "      class indexvalue (msg) {}\n"\
  "      class indexop (msg) {}\n"\
  "    }\n"\
  "    class invkey (msg) {\n"\
  "      class keyvalue (msg) {}\n"\
  "      class keyop (msg) {}\n"\
  "    }\n"\
  "    class invcall (msg) {\n"\
  "      class callop (msg) {}\n"\
  "      class partype (msg) {}\n"\
  "      class invresult (msg) {}\n"\
  "      class invinput (msg) {}\n"\
  "      class invfmt (msg) {}\n"\
  "      class eof (msg) {}\n"\
  "      class parnumber (msg) {}\n"\
  "      class syncthreadcall (msg) {}\n"\
  "      class syserror (msg) {\n"\
  "        class eaccess (msg) {}\n"\
  "        class eagain (msg) {}\n"\
  "        class ebadf (msg) {}\n"\
  "        class ebusy (msg) {}\n"\
  "        class echild (msg) {}\n"\
  "        class edeadlk (msg) {}\n"\
  "        class edom (msg) {}\n"\
  "        class eexist (msg) {}\n"\
  "        class efault (msg) {}\n"\
  "        class efbig (msg) {}\n"\
  "        class eintr (msg) {}\n"\
  "        class einval (msg) {}\n"\
  "        class eio (msg) {}\n"\
  "        class eisdir (msg) {}\n"\
  "        class emfile (msg) {}\n"\
  "        class emlink (msg) {}\n"\
  "        class enametoolong (msg) {}\n"\
  "        class enfile (msg) {}\n"\
  "        class enodev (msg) {}\n"\
  "        class enoent (msg) {}\n"\
  "        class enoexec (msg) {}\n"\
  "        class enolck (msg) {}\n"\
  "        class enomem (msg) {}\n"\
  "        class enospc (msg) {}\n"\
  "        class enosys (msg) {}\n"\
  "        class enotdir (msg) {}\n"\
  "        class enotempty (msg) {}\n"\
  "        class enotty (msg) {}\n"\
  "        class enxio (msg) {}\n"\
  "        class eperm (msg) {}\n"\
  "        class epipe (msg) {}\n"\
  "        class erange (msg) {}\n"\
  "        class erofs (msg) {}\n"\
  "        class espipe (msg) {}\n"\
  "        class esrch (msg) {}\n"\
  "        class exdev (msg) {}\n"\
  "      }\n"\
  "      class systemcall (msg) {\n"\
  "        class noshell (msg) {}\n"\
  "        class systemfail (msg) {}\n"\
  "      }\n"\
  "      class invregexp (msg) {\n"\
  "        class ebrack (msg) {}\n"\
  "        class erange (msg) {}\n"\
  "        class ectype (msg) {}\n"\
  "        class eparen (msg) {}\n"\
  "        class esubreg (msg) {}\n"\
  "        class eend (msg) {}\n"\
  "        class eescape (msg) {}\n"\
  "        class badpat (msg) {}\n"\
  "        class esize (msg) {}\n"\
  "        class espace (msg) {}\n"\
  "      }\n"\
  "      class invparser (msg) {\n"\
  "        class invgrammar (msg) {}\n"\
  "        class invtoken (msgs) {}\n"\
  "        class pmemory (msgs) {}\n"\
  "      }\n"\
  "      class invextern (msg) {\n"\
  "        class noextern (msg) {}\n"\
  "        class libclose (msg) {}\n"\
  "        class noexternsupp (msg) {}\n"\
  "      }\n"\
  "      class invenvar (msg) {}\n"\
  "      class internal (msg) {}\n"\
  "    }\n"\
  "    class invaccess (msg) {\n"\
  "      class accessop (msg) {}\n"\
  "      class accessvalue (msg) {}\n"\
  "      class immutable (msg) {}\n"\
  "    }\n"\
  "    class deadlock (msg) {}\n"\
  "    class syncwait (msg) {}\n"\
  "  }\n"\
  "}\n"\
  "var final excepts = except ();\n"\
  "var final errors = excepts.error (nil);\n"\
  "var final signals = errors.signal (nil);\n"\
  "var final invops = errors.invop (nil);\n"\
  "var final invindexes = errors.invindex (nil);\n"\
  "var final invkeys = errors.invkey (nil);\n"\
  "var final invcalls = errors.invcall (nil);\n"\
  "var final syserrors = invcalls.syserror (nil);\n"\
  "var final systemcalls = invcalls.systemcall (nil);\n"\
  "var final invregexps = invcalls.invregexp (nil);\n"\
  "var final invparsers = invcalls.invparser (nil);\n"\
  "var final invexterns = invcalls.invextern (nil);\n"\
  "var final invaccesses = errors.invaccess (nil);\n"\
  "var argv, env, final version, final main_thread, final curr_thread;\n"\
  "class anode (name, transl) {\n"\
  "  public name, transl;\n"\
  "}\n"\
  "class token (code) {public code;}\n"\
  "var final nil_anode = anode (\"$nil\");\n"\
  "var final error_anode = anode (\"$error\");\n"\
  "final class parser ()\n"\
  "{\n"\
  "  var ambiguous_p;\n"\
  "  public ambiguous_p;\n"\
  "  var grammar;\n"\
  "  private grammar;\n"\
  "\n"\
  "  func int_earley_parse_grammar(){} func int_earley_set_debug_level(){}\n"\
  "  private int_earley_parse_grammar, int_earley_set_debug_level;\n"\
  "  func int_earley_set_one_parse_flag (){}\n"\
  "  private int_earley_set_one_parse_flag;\n"\
  "  func int_earley_set_lookahead_level (){}\n"\
  "  private int_earley_set_lookahead_level;\n"\
  "  func int_earley_set_cost_flag (){}\n"\
  "  private int_earley_set_cost_flag;\n"\
  "  func int_earley_set_error_recovery_flag () {}\n"\
  "  private int_earley_set_error_recovery_flag;\n"\
  "  func int_earley_set_recovery_match (){} func int_earley_parse (){}\n"\
  "  private int_earley_set_recovery_match, int_earley_parse;\n"\
  "  func int_earley_create_grammar (){}\n"\
  "  private int_earley_create_grammar;\n"\
  "\n"\
  "  func set_grammar (descr, strict_p) {\n"\
  "    int_earley_parse_grammar (grammar, strict_p, descr);\n"\
  "  }\n"\
  "  func set_debug (level) {\n"\
  "    return int_earley_set_debug_level (grammar, level);\n"\
  "  }\n"\
  "  func set_one_parse (flag) {\n"\
  "    return int_earley_set_one_parse_flag (grammar, flag);\n"\
  "  }\n"\
  "  func set_lookahead (flag) {\n"\
  "    return int_earley_set_lookahead_level (grammar, flag);\n"\
  "  }\n"\
  "  func set_cost (flag) {\n"\
  "    return int_earley_set_cost_flag (grammar, flag);\n"\
  "  }\n"\
  "  func set_recovery (flag) {\n"\
  "    return int_earley_set_error_recovery_flag (grammar, flag);\n"\
  "  }\n"\
  "  func set_recovery_match (n_toks) {\n"\
  "    return int_earley_set_recovery_match (grammar, n_toks);\n"\
  "  }\n"\
  "  func parse (tokens, error_func) {\n"\
  "    var __i; // Trick to guarantee temp. vars to call error_func\n"\
  "    __i = 1 + 2 * (3 + 4 * (5 + 6 * 7));\n"\
  "    return int_earley_parse (grammar, tokens, error_func);\n"\
  "  }\n"\
  "\n"\
  "  grammar = int_earley_create_grammar ();\n"\
  "}\n"\
  "// Do not place any code inside file (it is only for internal purposes)\n"\
  "final class file (){\n"\
  "  private file_ptr; var file_ptr;\n"\
  "  if (__nogenerate__ != nil)\n"\
  "    throw invcalls.callop\n"\
  "            (\"File can be created only by functions open or popen\");}\n"\
  "var stdin = final file ();\n"\
  "var stdout = final file ();\n"\
  "var stderr = final file ();\n"\
  "var final __nogenerate__ = 0;\n"\
  "func max (){} func min (){}\n"\
  "func tolower (){} func toupper (){} func trans (){}\n"\
  "func eltype (){}\n"\
  "func keys (){}\n"\
  "func context (){} func inside (){}\n"\
  "func subv(){} func cmpv(){} func del(){} func ins(){}  func insv(){}\n"\
  "func rev(){} func sort(){}\n"\
  "func sqrt (){} func exp (){} func log (){} func log10(){} func pow(){} \n"\
  "func sin(){} func cos(){} func atan2(){} \n"\
  "func match(){} func gmatch(){} func sub(){} func gsub(){}\n"\
  "func split(){} var split_regex = \"[ \t]+\";\n"\
  "func srand(){} func rand(){} \n"\
  "func __process_errno__() {} \n"\
  "func readdir(){} func ftype () {}\n"\
  "func fun(){} func fgn () {} func fsize(){}\n"\
  "func fatime() {} func fmtime(){} func fctime () {}\n"\
  "func fumode() {} func fgmode(){} func fomode () {}\n"\
  "func time(){}\n"\
  "func strtime() {} var time_format = \"%a %b %d %H:%M:%S %Z %Y\";\n"\
  "func clock(){}\n"\
  "func gc() {} \n"\
  "func rename() {} func remove(){}\n"\
  "func mkdir() {} func rmdir(){}\n"\
  "func getcwd() {} func chdir(){}\n"\
  "func chumod() {} func chgmod() {} func chomod() {} func isatty(){}\n"\
  "func open() {} func close(){} func flush(){}\n"\
  "func popen() {} func pclose(){}\n"\
  "func tell() {} func seek() {}\n"\
  "func put(){} func putln(){}\n"\
  "func fput(){} func fputln(){} func sput(){} func sputln(){}\n"\
  "func putf (){} func fputf (){} func sputf (){}\n"\
  "func print(){} func println(){}\n"\
  "func fprint(){} func fprintln(){} func sprint(){} func sprintln(){}\n"\
  "func get(){} func getln(){} func getf(){}\n"\
  "func fget(){} func fgetln(){} func fgetf(){}\n"\
  "func scan(){} func scanln(){} func fscan(){} func fscanln(){}\n"\
  "func getpid(){} func getun(){} func geteun(){}\n"\
  "func getgn(){} func getegn(){} func getgroups(){}\n"\
  "func system(){}\n"\
  "func exit(){}\n"\
  "func __init__() {} __init__();"

/* The following must be defined in ENVIRONMENT (see above).  The
   variables are to be defined in set_environment_declarations. */

#define EXCEPT_NAME "except"
extern IR_node_t except_decl;

#define ERROR_NAME "error"
extern IR_node_t error_decl;

#define SIGNAL_NAME "signal"
extern IR_node_t signal_decl;

#define SIGINT_NAME "sigint"
extern IR_node_t sigint_decl;

#define SIGILL_NAME "sigill"
extern IR_node_t sigill_decl;

#define SIGABRT_NAME "sigabrt"
extern IR_node_t sigabrt_decl;

#define SIGFPE_NAME "sigfpe"
extern IR_node_t sigfpe_decl;

#define SIGTERM_NAME "sigterm"
extern IR_node_t sigterm_decl;

#define SIGSEGV_NAME "sigsegv"
extern IR_node_t sigsegv_decl;

#define INVENV_NAME "invenv"
extern IR_node_t invenv_decl;

#define INVOP_NAME "invop"
extern IR_node_t invop_decl;

#define OPTYPE_NAME "optype"
extern IR_node_t optype_decl;

#define OPVALUE_NAME "opvalue"
extern IR_node_t opvalue_decl;

#define INVINDEX_NAME "invindex"
extern IR_node_t invindex_decl;

#define INDEXTYPE_NAME "indextype"
extern IR_node_t indextype_decl;

#define INDEXVALUE_NAME "indexvalue"
extern IR_node_t indexvalue_decl;

#define INDEXOP_NAME "indexop"
extern IR_node_t indexop_decl;

#define INVKEY_NAME "invkey"
extern IR_node_t invkey_decl;

#define KEYVALUE_NAME "keyvalue"
extern IR_node_t keyvalue_decl;

#define KEYOP_NAME "keyop"
extern IR_node_t keyop_decl;

#define INVCALL_NAME "invcall"
extern IR_node_t invcall_decl;

#define CALLOP_NAME "callop"
extern IR_node_t callop_decl;

#define PARTYPE_NAME "partype"
extern IR_node_t partype_decl;

#define INVRESULT_NAME "invresult"
extern IR_node_t invresult_decl;

#define INVINPUT_NAME "invinput"
extern IR_node_t invinput_decl;

#define INVFMT_NAME "invfmt"
extern IR_node_t invfmt_decl;

#define EOF_NAME "eof"
extern IR_node_t eof_decl;

#define PARNUMBER_NAME "parnumber"
extern IR_node_t parnumber_decl;

#define SYNCTHREADCALL_NAME "syncthreadcall"
extern IR_node_t syncthreadcall_decl;

#define SYSERROR_NAME "syserror"
extern IR_node_t syserror_decl;

#define EACCESS_NAME "eaccess"
extern IR_node_t eaccess_decl;

#define EAGAIN_NAME "eagain"
extern IR_node_t eagain_decl;

#define EBADF_NAME "ebadf"
extern IR_node_t ebadf_decl;

#define EBUSY_NAME "ebusy"
extern IR_node_t ebusy_decl;

#define ECHILD_NAME "echild"
extern IR_node_t echild_decl;

#define EDEADLK_NAME "edeadlk"
extern IR_node_t edeadlk_decl;

#define EDOM_NAME "edom"
extern IR_node_t edom_decl;

#define EEXIST_NAME "eexist"
extern IR_node_t eexist_decl;

#define EFAULT_NAME "efault"
extern IR_node_t efault_decl;

#define EFBIG_NAME "efbig"
extern IR_node_t efbig_decl;

#define EINTR_NAME "eintr"
extern IR_node_t eintr_decl;

#define EINVAL_NAME "einval"
extern IR_node_t einval_decl;

#define EIO_NAME "eio"
extern IR_node_t eio_decl;

#define EISDIR_NAME "eisdir"
extern IR_node_t eisdir_decl;

#define EMFILE_NAME "emfile"
extern IR_node_t emfile_decl;

#define EMLINK_NAME "emlink"
extern IR_node_t emlink_decl;

#define ENAMETOOLONG_NAME "enametoolong"
extern IR_node_t enametoolong_decl;

#define ENFILE_NAME "enfile"
extern IR_node_t enfile_decl;

#define ENODEV_NAME "enodev"
extern IR_node_t enodev_decl;

#define ENOENT_NAME "enoent"
extern IR_node_t enoent_decl;

#define ENOEXEC_NAME "enoexec"
extern IR_node_t enoexec_decl;

#define ENOLCK_NAME "enolck"
extern IR_node_t enolck_decl;

#define ENOMEM_NAME "enomem"
extern IR_node_t enomem_decl;

#define ENOSPC_NAME "enospc"
extern IR_node_t enospc_decl;

#define ENOSYS_NAME "enosys"
extern IR_node_t enosys_decl;

#define ENOTDIR_NAME "enotdir"
extern IR_node_t enotdir_decl;

#define ENOTEMPTY_NAME "enotempty"
extern IR_node_t enotempty_decl;

#define ENOTTY_NAME "enotty"
extern IR_node_t enotty_decl;

#define ENXIO_NAME "enxio"
extern IR_node_t enxio_decl;

#define EPERM_NAME "eperm"
extern IR_node_t eperm_decl;

#define EPIPE_NAME "epipe"
extern IR_node_t epipe_decl;

#define ERANGE_NAME "erange"
extern IR_node_t erange_decl;

#define EROFS_NAME "erofs"
extern IR_node_t erofs_decl;

#define ESPIPE_NAME "espipe"
extern IR_node_t espipe_decl;

#define ESRCH_NAME "esrch"
extern IR_node_t esrch_decl;

#define EXDEV_NAME "exdev"
extern IR_node_t exdev_decl;

#define SYSTEMCALL_NAME "systemcall"
extern IR_node_t systemcall_decl;

#define NOSHELL_NAME "noshell"
extern IR_node_t noshell_decl;

#define SYSTEMFAIL_NAME "systemfail"
extern IR_node_t systemfail_decl;

#define INVREGEXP_NAME "invregexp"
extern IR_node_t invregexp_decl;

#define EBRACK_NAME "ebrack"
extern IR_node_t ebrack_decl;

#define REG_ERANGE_NAME "erange"
extern IR_node_t reg_erange_decl;

#define ECTYPE_NAME "ectype"
extern IR_node_t ectype_decl;

#define EPAREN_NAME "eparen"
extern IR_node_t eparen_decl;

#define ESUBREG_NAME "esubreg"
extern IR_node_t esubreg_decl;

#define EEND_NAME "eend"
extern IR_node_t eend_decl;

#define EESCAPE_NAME "eescape"
extern IR_node_t eescape_decl;

#define BADPAT_NAME "badpat"
extern IR_node_t badpat_decl;

#define ESIZE_NAME "esize"
extern IR_node_t esize_decl;

#define ESPACE_NAME "espace"
extern IR_node_t espace_decl;

#define INVPARSER_NAME "invparser"
extern IR_node_t invparser_decl;

#define INVGRAMMAR_NAME "invgrammar"
extern IR_node_t invgrammar_decl;

#define INVTOKEN_NAME "invtoken"
extern IR_node_t invtoken_decl;

#define PMEMORY_NAME "pmemory"
extern IR_node_t pmemory_decl;

#define INVEXTERN_NAME "invextern"
extern IR_node_t invextern_decl;

#define NOEXTERN_NAME "noextern"
extern IR_node_t noextern_decl;

#define LIBCLOSE_NAME "libclose"
extern IR_node_t libclose_decl;

#define NOEXTERNSUPP_NAME "noexternsupp"
extern IR_node_t noexternsupp_decl;

#define INVENVAR_NAME "invenvar"
extern IR_node_t invenvar_decl;

#define INTERNAL_NAME "internal"
extern IR_node_t internal_decl;

#define INVACCESS_NAME "invaccess"
extern IR_node_t invaccess_decl;

#define ACCESSOP_NAME "accessop"
extern IR_node_t accessop_decl;

#define ACCESSVALUE_NAME "accessvalue"
extern IR_node_t accessvalue_decl;

#define IMMUTABLE_NAME "immutable"
extern IR_node_t immutable_decl;

#define DEADLOCK_NAME "deadlock"
extern IR_node_t deadlock_decl;

#define SYNCWAIT_NAME "syncwait"
extern IR_node_t syncwait_decl;

#define EXCEPTS_NAME "excepts"
extern IR_node_t excepts_decl;

#define ERRORS_NAME "errors"
extern IR_node_t errors_decl;

#define SIGNALS_NAME "signals"
extern IR_node_t signals_decl;

#define INVOPS_NAME "invops"
extern IR_node_t invops_decl;

#define INVINDEXES_NAME "invindexes"
extern IR_node_t invindexes_decl;

#define INVKEYS_NAME "invkeys"
extern IR_node_t invkeys_decl;

#define SYSERRORS_NAME "syserrors"
extern IR_node_t syserrors_decl;

#define INVCALLS_NAME "invcalls"
extern IR_node_t invcalls_decl;

#define SYSTEMCALLS_NAME "systemcalls"
extern IR_node_t systemcalls_decl;

#define INVREGEXPS_NAME "invregexps"
extern IR_node_t invregexps_decl;

#define INVPARSERS_NAME "invparsers"
extern IR_node_t invparsers_decl;

#define INVEXTERNS_NAME "invexterns"
extern IR_node_t invexterns_decl;

#define INVACCESSES_NAME "invaccesses"
extern IR_node_t invaccesses_decl;

#define ARGV_NAME "argv"
extern IR_node_t argv_decl;

#define ENV_NAME "env"
extern IR_node_t env_decl;

#define VERSION_NAME "version"
extern IR_node_t version_decl;

#define MAIN_THREAD_NAME "main_thread"
extern IR_node_t main_thread_decl;

#define CURR_THREAD_NAME "curr_thread"
extern IR_node_t curr_thread_decl;

#define ANODE_NAME "anode"
extern IR_node_t anode_decl;

#define TOKEN_NAME "token"
extern IR_node_t token_decl;

#define CODE_NAME "code"
extern IR_node_t code_decl;

#define NIL_ANODE_NAME "nil_anode"
extern IR_node_t nil_anode_decl;

#define ERROR_ANODE_NAME "error_anode"
extern IR_node_t error_anode_decl;

#define PARSER_NAME "parser"
extern IR_node_t parser_decl;

#define AMBIGUOUS_P_NAME "ambiguous_p"
extern IR_node_t ambiguous_p_decl;

#define INT_EARLEY_PARSE_GRAMMAR_NAME "int_earley_parse_grammar"
extern IR_node_t int_earley_parse_grammar_decl;

#define INT_EARLEY_SET_DEBUG_LEVEL_NAME "int_earley_set_debug_level"
extern IR_node_t int_earley_set_debug_level_decl;

#define INT_EARLEY_SET_ONE_PARSE_FLAG_NAME "int_earley_set_one_parse_flag"
extern IR_node_t int_earley_set_one_parse_flag_decl;

#define INT_EARLEY_SET_LOOKAHEAD_LEVEL_NAME "int_earley_set_lookahead_level"
extern IR_node_t int_earley_set_lookahead_level_decl;

#define INT_EARLEY_SET_COST_FLAG_NAME "int_earley_set_cost_flag"
extern IR_node_t int_earley_set_cost_flag_decl;

#define INT_EARLEY_SET_ERROR_RECOVERY_FLAG_NAME \
  "int_earley_set_error_recovery_flag"
extern IR_node_t int_earley_set_error_recovery_flag_decl;

#define INT_EARLEY_SET_RECOVERY_MATCH_NAME "int_earley_set_recovery_match"
extern IR_node_t int_earley_set_recovery_match_decl;

#define INT_EARLEY_PARSE_NAME "int_earley_parse"
extern IR_node_t int_earley_parse_decl;

#define INT_EARLEY_CREATE_GRAMMAR_NAME "int_earley_create_grammar"
extern IR_node_t int_earley_create_grammar_decl;

#define FILE_NAME "file"
extern IR_node_t file_decl;

#define FILE_PTR_NAME "file_ptr"
extern IR_node_t file_ptr_decl;

#define STDIN_NAME "stdin"
extern IR_node_t stdin_decl;

#define STDOUT_NAME "stdout"
extern IR_node_t stdout_decl;

#define STDERR_NAME "stderr"
extern IR_node_t stderr_decl;

#define MIN_NAME "min"
extern IR_node_t min_decl;

#define MAX_NAME "max"
extern IR_node_t max_decl;

#define TOLOWER_NAME "tolower"
extern IR_node_t tolower_decl;

#define TOUPPER_NAME "toupper"
extern IR_node_t toupper_decl;

#define TRANS_NAME "trans"
extern IR_node_t trans_decl;

#define ELTYPE_NAME "eltype"
extern IR_node_t eltype_decl;

#define KEYS_NAME "keys"
extern IR_node_t keys_decl;

#define CONTEXT_NAME "context"
extern IR_node_t context_decl;

#define INSIDE_NAME "inside"
extern IR_node_t inside_decl;

#define SUBV_NAME "subv"
extern IR_node_t subv_decl;

#define CMPV_NAME "cmpv"
extern IR_node_t cmpv_decl;

#define DEL_NAME "del"
extern IR_node_t del_decl;

#define INS_NAME "ins"
extern IR_node_t ins_decl;

#define INSV_NAME "insv"
extern IR_node_t insv_decl;

#define REV_NAME "rev"
extern IR_node_t rev_decl;

#define SORT_NAME "sort"
extern IR_node_t sort_decl;

#define SQRT_NAME "sqrt"
extern IR_node_t sqrt_decl;

#define EXP_NAME "exp"
extern IR_node_t exp_decl;

#define LOG_NAME "log"
extern IR_node_t log_decl;

#define LOG10_NAME "log10"
extern IR_node_t log10_decl;

#define POW_NAME "pow"
extern IR_node_t pow_decl;

#define SIN_NAME "sin"
extern IR_node_t sin_decl;

#define COS_NAME "cos"
extern IR_node_t cos_decl;

#define ATAN2_NAME "atan2"
extern IR_node_t atan2_decl;

#define MATCH_NAME "match"
extern IR_node_t match_decl;

#define GMATCH_NAME "gmatch"
extern IR_node_t gmatch_decl;

#define SUB_NAME "sub"
extern IR_node_t sub_decl;

#define GSUB_NAME "gsub"
extern IR_node_t gsub_decl;

#define SPLIT_NAME "split"
extern IR_node_t split_decl;

#define SPLIT_REGEX_NAME "split_regex"
extern IR_node_t split_regex_decl;

#define SRAND_NAME "srand"
extern IR_node_t srand_decl;

#define RAND_NAME "rand"
extern IR_node_t rand_decl;

#define PROCESS_ERRNO_NAME "__process_errno__"
extern IR_node_t process_errno_decl;

#define READDIR_NAME "readdir"
extern IR_node_t readdir_decl;

#define FTYPE_NAME "ftype"
extern IR_node_t ftype_decl;

#define FUN_NAME "fun"
extern IR_node_t fun_decl;

#define FGN_NAME "fgn"
extern IR_node_t fgn_decl;

#define FSIZE_NAME "fsize"
extern IR_node_t fsize_decl;

#define FATIME_NAME "fatime"
extern IR_node_t fatime_decl;

#define FMTIME_NAME "fmtime"
extern IR_node_t fmtime_decl;

#define FCTIME_NAME "fctime"
extern IR_node_t fctime_decl;

#define FUMODE_NAME "fumode"
extern IR_node_t fumode_decl;

#define FGMODE_NAME "fgmode"
extern IR_node_t fgmode_decl;

#define FOMODE_NAME "fomode"
extern IR_node_t fomode_decl;

#define TIME_NAME "time"
extern IR_node_t time_decl;

#define STRTIME_NAME "strtime"
extern IR_node_t strtime_decl;

#define TIME_FORMAT_NAME "time_format"
extern IR_node_t time_format_decl;

#define CLOCK_NAME "clock"
extern IR_node_t clock_decl;

#define GC_NAME "gc"
extern IR_node_t gc_decl;

#define RENAME_NAME "rename"
extern IR_node_t rename_decl;

#define REMOVE_NAME "remove"
extern IR_node_t remove_decl;

#define MKDIR_NAME "mkdir"
extern IR_node_t mkdir_decl;

#define RMDIR_NAME "rmdir"
extern IR_node_t rmdir_decl;

#define GETCWD_NAME "getcwd"
extern IR_node_t getcwd_decl;

#define CHDIR_NAME "chdir"
extern IR_node_t chdir_decl;

#define CHUMOD_NAME "chumod"
extern IR_node_t chumod_decl;

#define CHGMOD_NAME "chgmod"
extern IR_node_t chgmod_decl;

#define CHOMOD_NAME "chomod"
extern IR_node_t chomod_decl;

#define ISATTY_NAME "isatty"
extern IR_node_t isatty_decl;

#define OPEN_NAME "open"
extern IR_node_t open_decl;

#define CLOSE_NAME "close"
extern IR_node_t close_decl;

#define FLUSH_NAME "flush"
extern IR_node_t flush_decl;

#define POPEN_NAME "popen"
extern IR_node_t popen_decl;

#define PCLOSE_NAME "pclose"
extern IR_node_t pclose_decl;

#define TELL_NAME "tell"
extern IR_node_t tell_decl;

#define SEEK_NAME "seek"
extern IR_node_t seek_decl;

#define PUT_NAME "put"
extern IR_node_t put_decl;

#define PUTLN_NAME "putln"
extern IR_node_t putln_decl;

#define FPUT_NAME "fput"
extern IR_node_t fput_decl;

#define FPUTLN_NAME "fputln"
extern IR_node_t fputln_decl;

#define SPUT_NAME "sput"
extern IR_node_t sput_decl;

#define SPUTLN_NAME "sputln"
extern IR_node_t sputln_decl;

#define PUTF_NAME "putf"
extern IR_node_t putf_decl;

#define FPUTF_NAME "fputf"
extern IR_node_t fputf_decl;

#define SPUTF_NAME "sputf"
extern IR_node_t sputf_decl;

#define PRINT_NAME "print"
extern IR_node_t print_decl;

#define PRINTLN_NAME "println"
extern IR_node_t println_decl;

#define FPRINT_NAME "fprint"
extern IR_node_t fprint_decl;

#define FPRINTLN_NAME "fprintln"
extern IR_node_t fprintln_decl;

#define SPRINT_NAME "sprint"
extern IR_node_t sprint_decl;

#define SPRINTLN_NAME "sprintln"
extern IR_node_t sprintln_decl;

#define GET_NAME "get"
extern IR_node_t get_decl;

#define GETLN_NAME "getln"
extern IR_node_t getln_decl;

#define GETF_NAME "getf"
extern IR_node_t getf_decl;

#define FGET_NAME "fget"
extern IR_node_t fget_decl;

#define FGETLN_NAME "fgetln"
extern IR_node_t fgetln_decl;

#define FGETF_NAME "fgetf"
extern IR_node_t fgetf_decl;

#define SCAN_NAME "scan"
extern IR_node_t scan_decl;

#define SCANLN_NAME "scanln"
extern IR_node_t scanln_decl;

#define FSCAN_NAME "fscan"
extern IR_node_t fscan_decl;

#define FSCANLN_NAME "fscanln"
extern IR_node_t fscanln_decl;

#define GETPID_NAME "getpid"
extern IR_node_t getpid_decl;

#define GETPGID_NAME "getpgid"
extern IR_node_t getpgid_decl;

#define GETUN_NAME "getun"
extern IR_node_t getun_decl;

#define GETEUN_NAME "geteun"
extern IR_node_t geteun_decl;

#define GETGN_NAME "getgn"
extern IR_node_t getgn_decl;

#define GETEGN_NAME "getegn"
extern IR_node_t getegn_decl;

#define GETGROUPS_NAME "getgroups"
extern IR_node_t getgroups_decl;

#define SYSTEM_NAME "system"
extern IR_node_t system_decl;

#define EXIT_NAME "exit"
extern IR_node_t exit_decl;

#define INIT_NAME "__init__"
extern IR_node_t init_decl;



#line 4115 "d_ir.h"
#endif /* __IR_d_ir__ */
